/*
    ------------------------------------------------------------------------------------
    LICENSE:
    ------------------------------------------------------------------------------------
    This file is part of EVEmu: EVE Online Server Emulator
    Copyright 2006 - 2021 The EVEmu Team
    For the latest information visit https://evemu.dev
    ------------------------------------------------------------------------------------
    This program is free software; you can redistribute it and/or modify it under
    the terms of the GNU Lesser General Public License as published by the Free Software
    Foundation; either version 2 of the License, or (at your option) any later
    version.

    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
    FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
    this program; if not, write to the Free Software Foundation, Inc., 59 Temple
    Place - Suite 330, Boston, MA 02111-1307, USA, or go to
    http://www.gnu.org/copyleft/lesser.txt.
    ------------------------------------------------------------------------------------
    Author:        ozatomic
*/

#include "eve-server.h"

#include "PyServiceCD.h"
#include "account/InfoGatheringMgr.h"

PyCallable_Make_InnerDispatcher(InfoGatheringMgr)

InfoGatheringMgr::InfoGatheringMgr(PyServiceMgr *mgr)
: PyService(mgr, "infoGatheringMgr"),
  m_dispatch(new Dispatcher(this))
{
    _SetCallDispatcher(m_dispatch);

    PyCallable_REG_CALL(InfoGatheringMgr, GetStateAndConfig);
    PyCallable_REG_CALL(InfoGatheringMgr, LogInfoEventsFromClient);

}

InfoGatheringMgr::~InfoGatheringMgr() {
    delete m_dispatch;
}

PyResult InfoGatheringMgr::Handle_GetStateAndConfig(PyCallArgs &call) {

    PyDict *rsp = new PyDict();

    rsp->SetItemString("clientWorkerInterval", new PyInt(600000)); //Default From packetlogs is 600000
    rsp->SetItemString("isEnabled", new PyInt(0)); //0 = Disabled, 1 = Enabled. Set to 0 becuase jsut gettting rid of exception.

    rsp->SetItemString("infoTypeAggregates", PyStatic.NewNone());
    rsp->SetItemString("infoTypesOncePerRun", PyStatic.NewNone());
    rsp->SetItemString("infoTypeParameters", PyStatic.NewNone());

    PyList *infoTypes = new PyList();
    infoTypes->AddItemInt(999); //Adding a value that was not in live so when its checks list it will always return false for now.

    rsp->SetItemString("infoTypes", new PyObjectEx_Type1( new PyToken("__builtin__.set"), new_tuple(infoTypes)));

    return new PyObject( "util.KeyVal", rsp );
}

PyResult InfoGatheringMgr::Handle_LogInfoEventsFromClient(PyCallArgs &call) {
    return PyStatic.NewNone();
}

/*
 * ==================== Sent from Client 95 bytes
 *
 * [PyObjectData Name: macho.CallReq]
 *  [PyTuple 7 items]
 *    [PyInt 6]
 *    [PyObjectData Name: macho.MachoAddress]
 *      [PyTuple 4 items]
 *        [PyInt 2]
 *        [PyInt 0]
 *        [PyIntegerVar 103]
 *        [PyNone]
 *    [PyObjectData Name: macho.MachoAddress]
 *      [PyTuple 3 items]
 *        [PyInt 8]
 *        [PyString "infoGatheringMgr"]
 *        [PyNone]
 *    [PyInt 5654387]
 *    [PyTuple 1 items]
 *      [PyTuple 2 items]
 *        [PyInt 0]
 *        [PySubStream 33 bytes]
 *          [PyTuple 4 items]
 *            [PyInt 1]
 *            [PyString "GetStateAndConfig"]
 *            [PyTuple 0 items]
 *            [PyDict 1 kvp]
 *              [PyString "machoVersion"]
 *              [PyInt 1]
 *    [PyNone]
 *    [PyNone]
 *
 *
 * ==================== Sent from Server 463 bytes [Compressed]
 *
 * [PyObjectData Name: macho.CallRsp]
 *  [PyTuple 7 items]
 *    [PyInt 7]
 *    [PyObjectData Name: macho.MachoAddress]
 *      [PyTuple 3 items]
 *        [PyInt 8]
 *        [PyString "infoGatheringMgr"]
 *        [PyNone]
 *    [PyObjectData Name: macho.MachoAddress]
 *      [PyTuple 4 items]
 *        [PyInt 2]
 *        [PyIntegerVar 77871000001103]
 *        [PyIntegerVar 103]
 *        [PyNone]
 *    [PyInt 5654387]
 *    [PyTuple 1 items]
 *      [PySubStream 907 bytes]
 *        [PyObjectData Name: util.KeyVal]
 *          [PyDict 6 kvp]
 *            [PyString "clientWorkerInterval"]
 *            [PyInt 600000]
 *            [PyString "isEnabled"]
 *            [PyInt 1]
 *            [PyString "infoTypeAggregates"]
 *            [PyDict 19 kvp]
 *              [PyInt 2]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 3 items]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *              [PyInt 3]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 3 items]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 6]
 *              [PyInt 4]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 3 items]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 6]
 *              [PyInt 5]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 3 items]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 6]
 *              [PyInt 6]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 4 items]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *                      [PyInt 4]
 *              [PyInt 7]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 1 items]
 *                      [PyInt 1]
 *              [PyInt 8]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 2 items]
 *                      [PyInt 1]
 *                      [PyInt 4]
 *              [PyInt 9]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 2 items]
 *                      [PyInt 1]
 *                      [PyInt 4]
 *              [PyInt 12]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 3 items]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *              [PyInt 13]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 3 items]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *              [PyInt 14]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 3 items]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *              [PyInt 17]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 1 items]
 *                      [PyInt 1]
 *              [PyInt 18]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 2 items]
 *                      [PyInt 1]
 *                      [PyInt 3]
 *              [PyInt 19]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 2 items]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *              [PyInt 20]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 2 items]
 *                      [PyInt 1]
 *                      [PyInt 3]
 *              [PyInt 21]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 2 items]
 *                      [PyInt 1]
 *                      [PyInt 3]
 *              [PyInt 22]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 5 items]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *                      [PyInt 4]
 *                      [PyInt 5]
 *              [PyInt 25]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 1 items]
 *                      [PyInt 6]
 *              [PyInt 26]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 1 items]
 *                      [PyInt 1]
 *            [PyString "infoTypesOncePerRun"]
 *            [PyObjectEx Normal]
 *              [PyTuple 2 items]
 *                [PyToken __builtin__.set]
 *                [PyTuple 1 items]
 *                  [PyList 0 items]
 *            [PyString "infoTypeParameters"]
 *            [PyDict 22 kvp]
 *              [PyInt 2]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 5 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *                      [PyInt 4]
 *              [PyInt 3]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 5 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *                      [PyInt 6]
 *              [PyInt 4]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 5 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *                      [PyInt 6]
 *              [PyInt 5]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 5 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *                      [PyInt 6]
 *              [PyInt 6]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 6 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *                      [PyInt 4]
 *                      [PyInt 5]
 *              [PyInt 7]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 3 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 3]
 *              [PyInt 8]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 4 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 3]
 *                      [PyInt 4]
 *              [PyInt 9]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 4 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 3]
 *                      [PyInt 4]
 *              [PyInt 12]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 5 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *                      [PyInt 4]
 *              [PyInt 13]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 5 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *                      [PyInt 9]
 *              [PyInt 14]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 5 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *                      [PyInt 9]
 *              [PyInt 15]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 6 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *                      [PyInt 4]
 *                      [PyInt 5]
 *              [PyInt 16]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 8 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *                      [PyInt 4]
 *                      [PyInt 5]
 *                      [PyInt 9]
 *                      [PyInt 10]
 *              [PyInt 17]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 5 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 3]
 *                      [PyInt 4]
 *                      [PyInt 5]
 *              [PyInt 18]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 4 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 3]
 *                      [PyInt 4]
 *              [PyInt 19]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 6 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *                      [PyInt 4]
 *                      [PyInt 5]
 *              [PyInt 20]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 4 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 3]
 *                      [PyInt 4]
 *              [PyInt 21]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 4 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 3]
 *                      [PyInt 4]
 *              [PyInt 22]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 7 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *                      [PyInt 4]
 *                      [PyInt 5]
 *                      [PyInt 9]
 *              [PyInt 24]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 4 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 2]
 *                      [PyInt 3]
 *              [PyInt 25]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 4 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 6]
 *                      [PyInt 9]
 *              [PyInt 26]
 *              [PyObjectEx Normal]
 *                [PyTuple 2 items]
 *                  [PyToken __builtin__.set]
 *                  [PyTuple 1 items]
 *                    [PyList 3 items]
 *                      [PyInt 0]
 *                      [PyInt 1]
 *                      [PyInt 3]
 *            [PyString "infoTypes"]
 *            [PyObjectEx Normal]
 *              [PyTuple 2 items]
 *                [PyToken __builtin__.set]
 *                [PyTuple 1 items]
 *                  [PyList 22 items]
 *                    [PyInt 2]
 *                    [PyInt 3]
 *                    [PyInt 4]
 *                    [PyInt 5]
 *                    [PyInt 6]
 *                    [PyInt 7]
 *                    [PyInt 8]
 *                    [PyInt 9]
 *                    [PyInt 12]
 *                    [PyInt 13]
 *                    [PyInt 14]
 *                    [PyInt 15]
 *                    [PyInt 16]
 *                    [PyInt 17]
 *                    [PyInt 18]
 *                    [PyInt 19]
 *                    [PyInt 20]
 *                    [PyInt 21]
 *                    [PyInt 22]
 *                    [PyInt 24]
 *                    [PyInt 25]
 *                    [PyInt 26]
 *    [PyNone]
 *    [PyNone]
 *
 */
