/*
    ------------------------------------------------------------------------------------
    LICENSE:
    ------------------------------------------------------------------------------------
    This file is part of EVEmu: EVE Online Server Emulator
    Copyright 2006 - 2021 The EVEmu Team
    For the latest information visit https://evemu.dev
    ------------------------------------------------------------------------------------
    This program is free software; you can redistribute it and/or modify it under
    the terms of the GNU Lesser General Public License as published by the Free Software
    Foundation; either version 2 of the License, or (at your option) any later
    version.

    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
    FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License along with
    this program; if not, write to the Free Software Foundation, Inc., 59 Temple
    Place - Suite 330, Boston, MA 02111-1307, USA, or go to
    http://www.gnu.org/copyleft/lesser.txt.
    ------------------------------------------------------------------------------------
    Author:        Captnoord
    Updates:    Allan
*/


#ifndef AttributeEnum_h__
#define AttributeEnum_h__

enum EveAttrEnum {
    AttrOnline = 2,
    AttrDamage = 3,
    AttrMass = 4,
    AttrStartTime = 5,                                  // filetime of when event started (or will start)
    AttrCapacitorNeed = 6,                              //dischargeAttributeID in dgmEffects
    AttrMinRange = 8,                                   // only used by 22476 which is non-published
    AttrHP = 9,
    AttrPowerOutput = 11,
    AttrLowSlots = 12,
    AttrMedSlots = 13,
    AttrHiSlots = 14,
    AttrPowerLoad = 15,
    AttrCapacitorCharge = 18,
    AttrPowerToSpeed = 19,
    AttrSpeedFactor = 20,
    AttrWarpFactor = 21,
    AttrWarpInhibitor = 29,
    AttrPower = 30,                                     //required by this item for fitting
    AttrMaxArmor = 31,
    AttrBreakPoint = 32,
    AttrMaxVelocity = 37,
    AttrCapacity = 38,                                  //cargohold capacity
    AttrDamageHP = 39,
    AttrSlots = 47,
    AttrCpuOutput = 48,
    AttrCpuLoad = 49,
    AttrCpu = 50,                                       //required by this item for fitting
    AttrSpeed = 51,                                     //durationAttributeID in dgmEffects
    AttrDamageResistance = 52,
    AttrMaxRange = 54,                                  //rangeAttributeID in dgmEffects
    AttrRechargeRate = 55,                              //Amount of time taken to fully recharge the capacitor.
    AttrChargeRate = 56,                                //Number of charges consumed per activation
    AttrTargetModule = 61,                              // module groupID targeted  - db has some data, but all are 0
    AttrAccuracyBonus = 63,
    AttrDamageMultiplier = 64,
    AttrArmorBonus = 65,
    AttrDurationBonus = 66,
    AttrCapacitorBonus = 67,
    AttrShieldBonus = 68,
    AttrRateBonus = 69,
    AttrInetia = 70,                                    // listed in db as "agility", this is actually Inertia, where agility is an internal variable
    AttrCapacityBonus = 72,
    AttrDuration = 73,                                  //durationAttributeID in dgmEffects
    AttrHpToCapacity = 75,                              // dunno what this is for
    AttrMaxTargetRange = 76,                            // npc dont have this
    AttrMiningAmount = 77,
    AttrScanSpeed = 79,                                 //ship scanning speed in milliseconds
    AttrSpeedBonus = 80,
    AttrHpFactor = 81,
    AttrStructureBonus = 82,
    AttrStructureDamageAmount = 83,
    AttrArmorDamageAmount = 84,
    AttrShieldTransferRange = 87,                       //rangeAttributeID in dgmEffects
    AttrShieldDrainAmount = 88,
    AttrShieldDrainRange = 89,                          // not used.  no items with this attribute
    AttrPowerTransferAmount = 90,
    AttrPowerTransferRange = 91,                        //rangeAttributeID in dgmEffects
    AttrKineticDampeningFieldStrength = 92,
    AttrKineticDampeningFieldBonus = 93,
    AttrEnergyReflectionStrength = 95,
    AttrEnergyReflectionBonus = 96,
    AttrEnergyDestabilizationAmount = 97,
    AttrEnergyDestabilizationRange = 98,                //rangeAttributeID in dgmEffects
    AttrEmpFieldRange = 99,                             //rangeAttributeID in dgmEffects       -smartbombs
    AttrLauncherSlotsLeft = 101,
    AttrTurretSlotsLeft = 102,
    AttrWarpScrambleRange = 103,                        //rangeAttributeID in dgmEffects
    AttrWarpScrambleStatus = 104,
    AttrWarpScrambleStrength = 105,
    AttrDroneBaySlotsLeft = 106,
    AttrExplosionRange = 107,
    AttrDetonationRange = 108,
    AttrKineticDamageResonance = 109,
    AttrThermalDamageResonance = 110,
    AttrExplosiveDamageResonance = 111,
    AttrEnergyDamageAbsorptionFactor = 112,
    AttrEmDamageResonance = 113,
    AttrEmDamage = 114,
    AttrExplosiveDamage = 116,
    AttrKineticDamage = 117,
    AttrThermalDamage = 118,
    AttrWeaponRangeMultiplier = 120,
    AttrPowerOutputBonus = 121,
    AttrArmorPiercingChance = 122,
    AttrShieldPiercingChance = 123,
    AttrMainColor = 124,
    AttrShipScanRange = 125,                            //rangeAttributeID in dgmEffects
    AttrCargoScanRange = 126,                           //rangeAttributeID in dgmEffects
    AttrAmmoLoaded = 127,
    // chargeSize is for ammo/crystals only (having this defined sets size tag on item pic)
    AttrChargeSize = 128,                               //1 = small, 2 = medium, 3 = large, 4 = xlarge
    AttrMaxPassengers = 129,
    AttrThermalDamageResonanceMultiplier = 130,
    AttrKineticDamageResonanceMultiplier = 131,
    AttrExplosiveDamageResonanceMultiplier = 132,
    AttrEmDamageResonanceMultiplier = 133,
    AttrShieldRechargeRateMultiplier = 134,
    AttrModuleSize = 135,
    AttrUniformity = 136,
    AttrLauncherGroup = 137,
    AttrEmDamageBonus = 138,
    AttrExplosiveDamageBonus = 139,
    AttrKineticDamageBonus = 140,
    AttrThermalDamageBonus = 141,
    AttrEcmBurstRange = 142,                            //rangeAttributeID in dgmEffects      -ecm burst
    AttrTargetHostileRange = 143,
    AttrCapacitorRechargeRateMultiplier = 144,
    AttrPowerOutputMultiplier = 145,
    AttrShieldCapacityMultiplier = 146,
    AttrCapacitorCapacityMultiplier = 147,
    AttrArmorHPMultiplier = 148,
    AttrCargoCapacityMultiplier = 149,
    AttrStructureHPMultiplier = 150,
    AttrAgilityBonus = 151,
    AttrMaxPassengersBonus = 152,
    AttrWarpCapacitorNeed = 153,
    AttrProximityRange = 154,
    AttrIncapacitationRatio = 156,
    AttrOrbitRange = 157,
    AttrFalloff = 158,                                  //falloffAttributeID in dgmEffects
    AttrTrackingSpeed = 160,                            //trackingSpeedAttributeID in dgmEffects
    AttrVolume = 161,
    AttrRadius = 162,
    AttrDummyDuration = 163,
    AttrCharisma = 164,
    AttrIntelligence = 165,
    AttrMemory = 166,
    AttrPerception = 167,
    AttrWillpower = 168,
    AttrAgilityMultiplier = 169,
    AttrCustomCharismaBonus = 170,
    AttrCustomWillpowerBonus = 171,
    AttrCustomPerceptionBonus = 172,
    AttrCustomMemoryBonus = 173,
    AttrCustomIntelligenceBonus = 174,
    AttrCharismaBonus = 175,
    AttrIntelligenceBonus = 176,
    AttrMemoryBonus = 177,
    AttrPerceptionBonus = 178,
    AttrWillpowerBonus = 179,
    AttrPrimaryAttribute = 180,
    AttrSecondaryAttribute = 181,
    AttrRequiredSkill1 = 182,
    AttrRequiredSkill2 = 183,
    AttrRequiredSkill3 = 184,
    AttrAttributePoints = 185,
    AttrWarpCapacitorNeedMultiplier = 186,
    AttrRepairCostMultiplier = 187,
    AttrCargoScanResistance = 188,
    AttrTargetGroup = 189,
    AttrCorporationMemberLimit = 190,
    AttrCorporationMemberBonus = 191,
    AttrMaxLockedTargets = 192,
    AttrMaxAttackTargets = 193,
    AttrJammingResistance = 194,
    AttrRaceID = 195,
    AttrManufactureSlotLimit = 196,
    AttrSurveyScanRange = 197,
    AttrCpuMultiplier = 202,
    AttrMiningDurationMultiplier = 203,
    AttrSpeedMultiplier = 204,
    AttrAccuracyMultiplier = 205,
    AttrMiningAmountMultiplier = 207,
    AttrScanRadarStrength = 208,                        //Radar Site (scanning/exploration)
    AttrScanLadarStrength = 209,                        //Ladar Site (scanning/exploration)
    AttrScanMagnetometricStrength = 210,                //Magnetometric Site (scanning/exploration)
    AttrScanGravimetricStrength = 211,                  //Gravimetric Site (scanning/exploration)
    AttrMissileDamageMultiplier = 212,
    AttrMissileDamageMultiplierBonus = 213,
    AttrCapacitorNeedMultiplier = 216,
    AttrPropulsionGraphicID = 217,
    AttrBlueprintResearchTimeMultiplier = 218,
    AttrManufactureTimeMultiplier = 219,
    AttrBlueprintResearchTimeMultiplierBonus = 220,
    AttrBlueprintManufactureTimeMultiplier = 221,
    AttrBlueprintManufactureTimeMultiplierBonus = 222,
    AttrCharismaSkillTrainingTimeMultiplier = 223,
    AttrIntelligenceSkillTrainingTimeMultiplier = 224,
    AttrMemorySkillTrainingTimeMultiplier = 225,
    AttrPerceptionSkillTrainingTimeMultiplier = 226,
    AttrWillpowerSkillTrainingTimeMultiplier = 227,
    AttrCharismaSkillTrainingTimeMultiplierBonus = 228,
    AttrIntelligenceSkillTrainingTimeMultiplierBonus = 229,
    AttrMemorySkillTrainingTimeMultiplierBonus = 230,
    AttrPerceptionSkillTrainingTimeMultiplierBonus = 231,
    AttrWillpowerSkillTrainingTimeMultiplierBonus = 232,
    AttrMaxLockedTargetsBonus = 235,
    AttrMaxAttackTargetsBonus = 236,
    AttrMaxTargetRangeMultiplier = 237,
    AttrScanGravimetricStrengthBonus = 238,
    AttrScanLadarStrengthBonus = 239,
    AttrScanMagnetometricStrengthBonus = 240,
    AttrScanRadarStrengthBonus = 241,
    AttrScanSpeedMultiplier = 242,
    AttrMaxRangeMultiplier = 243,
    AttrTrackingSpeedMultiplier = 244,
    AttrGfxTurretID = 245,
    AttrGfxBoosterID = 246,
    AttrEntityAttackRange = 247,
    AttrEntityLootValueMin = 248,
    AttrEntityLootValueMax = 249,
    AttrEntityLootCountMin = 250,
    AttrEntityLootCountMax = 251,
    AttrEntitySecurityStatusKillBonus = 252,
    AttrEntitySecurityStatusAggressionBonus = 253,
    AttrMinLootCount = 254,
    AttrMaxLootCount = 256,
    AttrEntityFollowRange = 257,
    AttrMinLootValue = 258,
    AttrMaxLootValue = 259,
    AttrAttackRange = 260,
    AttrKillStatusModifier = 261,
    AttrAttackStatusModifier = 262,
    AttrShieldCapacity = 263,
    AttrShieldCharge = 264,
    AttrArmorHP = 265,
    AttrArmorDamage = 266,
    AttrArmorEmDamageResonance = 267,
    AttrArmorExplosiveDamageResonance = 268,
    AttrArmorKineticDamageResonance = 269,
    AttrArmorThermalDamageResonance = 270,
    AttrShieldEmDamageResonance = 271,
    AttrShieldExplosiveDamageResonance = 272,
    AttrShieldKineticDamageResonance = 273,
    AttrShieldThermalDamageResonance = 274,
    AttrSkillTimeConstant = 275,
    AttrSkillPoints = 276,
    AttrRequiredSkill1Level = 277,
    AttrRequiredSkill2Level = 278,
    AttrRequiredSkill3Level = 279,
    AttrSkillLevel = 280,
    AttrExplosionDelay = 281,               // in ms
    AttrLauncherCapacityMultiplier = 282,
    AttrDroneCapacity = 283,
    AttrExcludeGangMembers = 284,
    AttrExcludeCorporationMembers = 285,
    AttrExcludeHostiles = 286,
    AttrKDmgBonus = 287,
    AttrShipCPUBonus = 288,
    AttrTurretDamageBonus = 289,
    AttrSkillTurretDmgBonus = 290,
    AttrCpuskillBonus = 291,
    AttrDamageMultiplierBonus = 292,
    AttrRofBonus = 293,
    AttrRangeSkillBonus = 294,
    AttrAbPowerBonus = 295,
    AttrAcPowerBonus = 296,
    AttrAfPowerBonus = 297,
    AttrAtPowerBonus = 298,
    AttrCbTRangeBonus = 299,
    AttrCcTRangeBonus = 300,
    AttrCfTRangeBonus = 301,
    AttrCiTRangeBonus = 302,
    AttrAiPowerBonus = 303,
    AttrCtTRangeBonus = 304,
    AttrGbCpuBonus = 305,
    AttrMaxVelocityBonus = 306,
    AttrScannerDurationBonus = 307,
    AttrScanspeedBonus = 308,
    AttrMaxTargetRangeBonus = 309,
    AttrCpuNeedBonus = 310,
    AttrMaxTargetBonus = 311,
    AttrDurationSkillBonus = 312,
    AttrPowerEngineeringOutputBonus = 313,
    AttrCapRechargeBonus = 314,
    AttrVelocityBonus = 315,
    AttrCorpMemberBonus = 316,
    AttrCapNeedBonus = 317,
    AttrSpeedFBonus = 318,
    AttrWarpCapacitorNeedBonus = 319,
    AttrPowerUseBonus = 320,
    AttrBurstSpeed = 321,
    AttrBurstSpeedMutator = 322,
    AttrPowerNeedBonus = 323,
    AttrBarrageDmgMutator = 324,
    AttrBarrageFalloffMutator = 325,
    AttrBarrageDmgMultiplier = 326,
    AttrHullHpBonus = 327,
    AttrBarrageFalloff = 328,                           //falloffAttributeID in dgmEffects
    AttrGangRofBonus = 329,
    AttrBoosterDuration = 330,
    AttrImplantness = 331,
    AttrBurstDmg = 332,
    AttrBurstDmgMutator = 333,
    AttrShipPowerBonus = 334,
    AttrArmorHpBonus = 335,
    AttrUniformityBonus = 336,
    AttrShieldCapacityBonus = 337,
    AttrRechargeratebonus = 338,
    AttrFalloffBonus = 349,
    AttrSkillTrainingTimeBonus = 350,
    AttrMaxRangeBonus = 351,
    AttrMaxActiveDrones = 352,
    AttrMaxActiveDroneBonus = 353,
    AttrMaxDroneBonus = 354,
    AttrNegotiationPercentage = 355,
    AttrDiplomacyBonus = 356,
    AttrFastTalkPercentage = 359,
    AttrConnectionsBonus = 360,
    AttrCriminalConnectionsBonus = 361,
    AttrSocialBonus = 362,
    AttrAmarrTechTimePercent = 363,
    AttrMinmatarTechTimePercent = 364,
    AttrGallenteTechTimePercent = 365,
    AttrCaldariTechTimePercent = 366,
    AttrProductionTimePercent = 367,
    AttrRefiningTimePercentage = 368,
    AttrManufactureCostMultiplier = 369,
    AttrAmarrTechMutator = 370,
    AttrCaldariTechMutator = 371,
    AttrGallenteTechMutator = 372,
    AttrProductionTimeMutator = 373,
    AttrMinmatarTechMutator = 374,
    AttrProductionCostMutator = 375,
    AttrRefiningTimePercent = 376,
    AttrRefiningTimeMutator = 377,
    AttrRefiningYieldPercentage = 378,
    AttrRefiningYieldMutator = 379,
    AttrMaxActiveFactory = 380,
    AttrMaxActiveFactories = 383,
    AttrMaxResearchGangSize = 384,
    AttrManufacturingTimeResearchSpeed = 385,
    AttrResearchCostPercent = 386,
    AttrCopySpeedPercent = 387,
    AttrFrigateConstructionCost = 388,
    AttrCruiserConstructionCost = 389,
    AttrIndustrialConstructionCost = 392,
    AttrBattleshipConstructionCost = 393,
    AttrTitanConstructionTime = 394,
    AttrStationConstructionTime = 395,
    AttrRepairCostPercent = 396,
    AttrReverseEngineeringChance = 397,
    AttrMineralNeedResearchSpeed = 398,
    AttrDuplicatingChance = 399,
    AttrMissileStandardVelocityPecent = 400,
    AttrCruiseMissileVelocityPercent = 401,
    AttrHeavyMissileSpeedPercent = 402,
    AttrRocketDmgPercent = 403,
    AttrTorpedoVelocityPercent = 404,
    AttrDefenderVelocityPercent = 405,
    AttrMissileFOFVelocityPercent = 406,
    AttrResearchGangSizeBonus = 407,
    AttrBattleshipConstructionTimeBonus = 408,
    AttrCruiserConstructionTimeBonus = 409,
    AttrFrigateConstructionTimeBonus = 410,
    AttrIndustrialConstructionTimeBonus = 411,
    AttrConnectionBonusMutator = 412,
    AttrCriminalConnectionsMutator = 413,
    AttrDiplomacyMutator = 414,
    AttrFastTalkMutator = 415,
    AttrEntityFlyRange = 416,
    AttrMaxNonRaceCorporationMembers = 417,
    AttrNonRaceCorporationMembersBonus = 418,
    AttrSkillPointsSaved = 419,
    AttrTrackingBonus = 420,
    AttrShieldRechargerateBonus = 421,
    AttrTechLevel = 422,
    AttrEntityDroneCount = 423,
    AttrCpuOutputBonus2 = 424,
    AttrCpuOutputBonus = 425,
    AttrHeavyDroneDamagePercent = 426,
    AttrHeavyDroneDamageBonus = 427,
    AttrMiningDroneAmountPercent = 428,
    AttrMiningDroneSpeedBonus = 429,
    AttrScoutDroneVelocityPercent = 430,
    AttrScoutDroneVelocityBonus = 431,
    AttrDefenderVelocityBonus = 432,
    AttrHeavyMissileDamageBonus = 433,
    AttrMiningAmountBonus = 434,
    AttrMaxGangModules = 435,
    AttrStandingIncreasePercent = 436,
    AttrNegotiationBonus = 437,
    AttrSocialMutator = 438,
    AttrTargetingSpeedBonus = 439,
    AttrManufacturingTimeBonus = 440,
    AttrTurretSpeeBonus = 441,
    AttrBarterDiscount = 442,
    AttrTradePremium = 443,
    AttrContrabandFencingChance = 444,
    AttrSmugglingChance = 445,
    AttrTradePremiumBonus = 446,
    AttrSmugglingChanceBonus = 447,
    AttrFencingChanceBonus = 448,
    AttrBarterDiscountBonus = 449,
    AttrManufacturingSlotBonus = 450,
    AttrManufactureCostBonus = 451,
    AttrCopySpeedBonus = 452,
    AttrBlueprintmanufactureTimeBonus = 453,
    AttrMutaton = 454,
    AttrLearningBonus = 455,
    AttrEntityEquipmentMin = 456,
    AttrEntityEquipmentMax = 457,
    AttrDroneControlDistance = 458,
    AttrDroneRangeBonus = 459,
    AttrShipBonusMF = 460,
    AttrSpecialAbilityBonus = 461,
    AttrShipBonusGF = 462,
    AttrShipBonusCF = 463,
    AttrShipBonusAF = 464,
    AttrEntityEquipmentGroupMax = 465,
    AttrEntityReactionFactor = 466,
    AttrMaxLaborotorySlots = 467,
    AttrMineralNeedResearchBonus = 468,
    AttrEntityBluePrintDropChance = 469,
    AttrLootRespawnTime = 470,
    AttrLaboratorySlotsBonus = 471,
    AttrStationTypeID = 472,
    AttrPrototypingBonus = 473,
    AttrInventionBonus = 474,
    AttrEntityAttackDelayMin = 475,
    AttrEntityAttackDelayMax = 476,
    AttrShipBonusAC = 478,
    AttrShieldRechargeRate = 479,
    AttrMaxEffectiveRange = 480,
    AttrEntityKillBounty = 481,
    AttrCapacitorCapacity = 482,
    AttrShieldUniformity = 484,
    AttrShipBonus2AF = 485,
    AttrShipBonusGC = 486,
    AttrShipBonusCC = 487,
    AttrShipVelocityBonusMC = 488,
    AttrShipBonusMC = 489,
    AttrShipBonusMB = 490,
    AttrShipBonusCB = 491,
    AttrShipBonusAB = 492,
    AttrShipBonusMI = 493,
    AttrShipBonusAI = 494,
    AttrShipBonusCI = 495,
    AttrShipBonusGI = 496,
    AttrEntityDefenderChance = 497,
    AttrDroneCapacityBonus = 499,
    AttrShipBonusGB = 500,
    AttrShipBonus2CB = 501,
    AttrEntityConvoyDroneMin = 502,
    AttrEntityConvoyDroneMax = 503,
    AttrEntityWarpScrambleChance = 504,                 // npcUsageChanceAttributeID in dgmEffects
    AttrWarpScrambleDuration = 505,
    AttrMissileLaunchDuration = 506,
    AttrEntityMissileTypeID = 507,
    AttrEntityCruiseSpeed = 508,
    AttrCargoScanFalloff = 509,                         //falloffAttributeID in dgmEffects
    AttrShipScanFalloff = 510,                          //falloffAttributeID in dgmEffects
    AttrShipScanResistance = 511,
    AttrModifyTargetSpeedChance = 512,                  // npcUsageChanceAttributeID in dgmEffects
    AttrModifyTargetSpeedDuration = 513,
    AttrModifyTargetSpeedRange = 514,                   //rangeAttributeID in dgmEffects        --npcs
    AttrModifyTargetSpeedCapacitorNeed = 515,
    AttrChassisType = 516,
    AttrFallofMultiplier = 517,
    AttrShipBonusMB2 = 518,
    AttrCloakingCapacitorNeedRatio = 519,
    AttrDamageCloudChance = 522,
    AttrArmorUniformity = 524,
    AttrStructureUniformity = 525,
    AttrReqResearchSkill = 526,
    AttrReqManufacturingSkill = 527,
    AttrReqManufacturingSkillLevel = 528,
    AttrReqResearchSkillLevel = 529,
    AttrReqManufacturingTool = 530,
    AttrReqResearchTool = 531,
    AttrReqResearchComponent = 532,
    AttrManufacturer_ID = 534,
    AttrInstalledMod = 535,
    AttrReqResearchComponetAmount = 536,
    AttrReqManufacturingComponent1Amount = 537,
    AttrReqManufacturingComponent2Amount = 538,
    AttrEntityStrength = 542,
    AttrDamageCloudChanceReduction = 543,
    AttrCloudEffectDelay = 544,
    AttrCloudDuration = 545,
    AttrDamageCloudType = 546,
    AttrMissileVelocityBonus = 547,
    AttrShieldBoostMultiplier = 548,
    AttrPowerIncrease = 549,
    AttrResistanceBonus = 550,
    AttrRocketVelocityPercent = 551,
    AttrSignatureRadius = 552,
    AttrMaxGangSizeBonus = 553,
    AttrSignatureRadiusBonus = 554,
    AttrCloakVelocityBonus = 555,
    AttrAnchoringDelay = 556,
    AttrMaxFlightTimeBonus = 557,
    AttrExplosionRangeBonus = 558,
    AttrInertia = 559,
    AttrCloakingTargetingDelay = 560,
    AttrShipBonusGB2 = 561,
    AttrEntityFactionLoss = 562,
    AttrEntitySecurityMaxGain = 563,
    AttrScanResolution = 564,
    AttrScanResolutionMultiplier = 565,
    AttrScanResolutionBonus = 566,
    AttrSpeedBoostFactor = 567,
    AttrEliteBonusInterceptor = 568,
    AttrEliteBonusCoverOps1 = 569,
    AttrEliteBonusBombers = 570,
    AttrEliteBonusGunships = 571,
    AttrEliteBonusdestroyers = 573,
    AttrEliteBonusBattlecruiser = 575,
    AttrSpeedBoostFactorCalc = 576,
    AttrSpeedBoostFactorCalc2 = 578,
    AttrTestForEggert = 579,
    AttrEntityChaseMaxDelay = 580,
    AttrEntityChaseMaxDelayChance = 581,
    AttrEntityChaseMaxDuration = 582,
    AttrEntityChaseMaxDurationChance = 583,
    AttrEntityMaxWanderRange = 584,
    AttrShipBonus2AB = 585,
    AttrShipBonus2GF = 586,
    AttrShipBonus2MF = 587,
    AttrShipBonus2CF = 588,
    AttrIsPlayerOwnable = 589,
    AttrGestaltBonus1 = 590,
    AttrDroneMaxVelocityBonus = 591,
    AttrCloakCapacitorBonus = 592,
    AttrDie = 594,
    AttrCapBoostMultipler = 595,
    AttrExplosionDelayBonus = 596,
    AttrEliteBonusEscorts = 597,
    AttrShipBonus3CB = 598,
    AttrWarpSpeedMultiplier = 600,
    AttrWarpSpeedBonus = 601,
    AttrLauncherGroup2 = 602,
    AttrLauncherGroup3 = 603,
    AttrChargeGroup1 = 604,
    AttrChargeGroup2 = 605,
    AttrChargeGroup3 = 606,
    AttrPowerNeedMultiplier = 608,
    AttrChargeGroup4 = 609,
    AttrChargeGroup5 = 610,
    AttrDurationMultiplier = 611,
    AttrBaseShieldDamage = 612,
    AttrBaseArmorDamage = 613,
    AttrCargoCapacityBonus = 614,
    AttrBoosterShieldBoostAmountPenalty = 616,
    AttrCloakingTargetingDelayBonus = 619,
    AttrOptimalSigRadius = 620,
    AttrTrackingSpeedAtOptimal = 621,
    AttrMassLimit = 622,
    AttrCloakingSlotsLeftSuper = 623,
    AttrWarpSBonus = 624,
    AttrBountyBonus = 625,
    AttrBountyMultiplier = 626,
    AttrBountySkillBonus = 627,
    AttrBountySkillMultiplyer = 628,
    AttrCargoGroup = 629,
    AttrEntityArmorRepairDuration = 630,
    AttrEntityArmorRepairAmount = 631,
    AttrInterceptorGF = 632,
    AttrMetaLevel = 633,
    AttrNewAgility = 634,
    AttrTurnAngle = 635,
    AttrEntityShieldBoostDuration = 636,
    AttrEntityShieldBoostAmount = 637,
    AttrEntityArmorRepairDelayChance = 638,
    AttrEntityShieldBoostDelayChance = 639,
    AttrEntityGroupRespawnChance = 640,
    AttrPrereqimplant = 641,
    AttrArmingTime = 643,
    AttrAimedLaunch = 644,
    AttrMissileEntityVelocityMultiplier = 645,
    AttrMissileEntityFlightTimeMultiplier = 646,
    AttrMissileEntityArmingTimeMultiplier = 647,        // no types use this
    AttrShieldTUNEBonus = 648,
    AttrCloakingCpuNeedBonus = 649,
    AttrMaxStructureDistance = 650,
    AttrDecloakFieldRange = 651,                //rangeAttributeID in dgmEffects
    AttrSignatureRadiusMultiplier = 652,
    AttrAoeVelocity = 653,
    AttrAoeCloudSize = 654,
    AttrAoeFalloff = 655,
    AttrShipBonus2AC = 656,
    AttrShipBonus2CC = 657,
    AttrShipBonus2GC = 658,
    AttrShipBonus2MC = 659,
    AttrImpactDamage = 660,
    AttrMaxDirectionalVelocity = 661,
    AttrMinTargetVelDmgMultiplier = 662,
    AttrMinMissileVelDmgMultiplier = 663,
    AttrMaxMissileVelocity = 664,
    AttrEntityChaseMaxDistance = 665,           // distance where entity will activate their speed mod
    AttrModuleShipGroup2 = 666,
    AttrModuleShipGroup3 = 667,
    AttrModuleShipGroup1 = 668,
    AttrModuleReactivationDelay = 669,
    AttrAreaOfEffectBonus = 670,
    AttrEntityCruiseSpeedMultiplier = 672,
    AttrEliteBonusGunship1 = 673,
    AttrEliteBonusGunship2 = 675,
    AttrUnanchoringDelay = 676,
    AttrOnliningDelay = 677,
    AttrEliteBonusLogistics1 = 678,
    AttrEliteBonusLogistics2 = 679,
    AttrShieldRadius = 680,
    AttrTypeContainerType1 = 681,
    AttrTypeContainerType2 = 682,
    AttrTypeContainerType3 = 683,
    AttrTypeContainerCapacity1 = 684,
    AttrTypeContainerCapacity2 = 685,
    AttrTypeContainerCapacity3 = 686,
    AttrOperationConsumptionRate = 687,
    AttrReinforcedConsumptionRate = 688,
    AttrPackageGraphicID = 689,
    AttrPackageRadius = 690,
    AttrTargetSwitchDelay = 691,
    AttrEliteBonusHeavyGunship1 = 692,
    AttrEliteBonusHeavyGunship2 = 693,
    AttrResourceReinforced1Type = 694,
    AttrResourceReinforced2Type = 695,
    AttrResourceReinforced3Type = 696,
    AttrResourceReinforced4Type = 697,
    AttrResourceReinforced5Type = 698,
    AttrResourceReinforced1Quantity = 699,
    AttrResourceReinforced2Quantity = 700,
    AttrResourceReinforced3Quantity = 701,
    AttrResourceReinforced4Quantity = 703,
    AttrResourceReinforced5Quantity = 704,
    AttrResourceOnline1Type = 705,
    AttrResourceOnline2Type = 706,
    AttrResourceOnline3Type = 707,
    AttrResourceOnline4Type = 708,
    AttrHarvesterType = 709,
    AttrHarvesterQuality = 710,
    AttrMoonAnchorDistance = 711,
    AttrUsageDamagePercent = 712,       //How much a charge used by this object is damaged each time the object operates.
    AttrConsumptionType = 713,
    AttrConsumptionQuantity = 714,
    AttrMaxOperationalDistance = 715,
    AttrMaxOperationalUsers = 716,
    AttrRefiningYieldMultiplier = 717,
    AttrOperationalDuration = 719,
    AttrRefineryCapacity = 720,
    AttrRefiningDelayMultiplier = 721,
    AttrPosControlTowerPeriod = 722,
    AttrContrabandDetectionChance = 723,
    AttrContrabandDetectionResistance = 724,
    AttrContrabandScanChance = 725,
    AttrMoonMiningAmount = 726,
    AttrDestroyerROFpenality = 727,
    AttrControlTowerLaserDamageBonus = 728,
    AttrShipBonus1MD = 729,
    AttrShipBonus1D = 732,
    AttrShipBonus2D = 733,
    AttrShipBonus1CD = 734,
    AttrShipBonus2CD = 735,
    AttrShipBonus1GD = 738,
    AttrShipBonus2GD = 739,
    AttrShipBonus2MD = 740,
    AttrShipBonus1BC = 741,
    AttrShipBonus2BC = 742,
    AttrShipBonus1CBC = 743,
    AttrShipBonus2CBC = 745,
    AttrShipBonus2GBC = 746,
    AttrShipBonus1GBC = 747,
    AttrShipBonus1MBC = 748,
    AttrShipBonus2MBC = 749,
    AttrControlTowerLaserOptimalBonus = 750,
    AttrControlTowerHybridOptimalBonus = 751,
    AttrControlTowerProjectileOptimalBonus = 752,
    AttrControlTowerProjectileFallOffBonus = 753,
    AttrControlTowerProjectileROFBonus = 754,
    AttrControlTowerMissileROFBonus = 755,
    AttrControlTowerMoonHarvesterCPUBonus = 756,
    AttrControlTowerSiloCapacityBonus = 757,
    AttrShipBonus1DF = 758,
    AttrShipBonus2DF = 759,
    AttrControlTowerLaserProximityRangeBonus = 760,
    AttrControlTowerProjectileProximityRangeBonus = 761,
    AttrControlTowerHybridProximityRangeBonus = 762,
    AttrMaxGroupActive = 763,
    AttrControlTowerEwRofBonus = 764,
    AttrScanRange = 765,                // range in AU
    AttrControlTowerHybridDamageBonus = 766,
    AttrTrackingSpeedBonus = 767,
    AttrMaxRangeBonus2 = 769,
    AttrControlTowerEwTargetSwitchDelayBonus = 770,
    AttrAmmoCapacity = 771,
    AttrEntityFlyRangeFactor = 772,
    AttrShipBonus1ORE = 773,
    AttrShipBonus2ORE = 774,
    AttrMiningCPUNeedBonus = 775,
    AttrStructureMissileVelocityBonus = 776,
    AttrStructureMissileDamageBonus = 777,
    AttrStructureMissileExplosionDelayBonus = 778,
    AttrEntityFlyRangeMultiplier = 779,
    AttrIceHarvestCycleBonus = 780,
    AttrSpecialisationAsteroidGroup = 781,
    AttrSpecialisationAsteroidYieldMultiplier = 782,
    AttrCrystalVolatilityChance = 783,
    AttrCrystalVolatilityDamage = 784,
    AttrUnfitCapCost = 785,     // The capacitor charge required to disengage this crystal from the unit it is installed in.
    AttrCrystalsGetDamaged = 786,
    AttrMinScanDeviation = 787,
    AttrMaxScanDeviation = 788,
    AttrSpecialtyMiningAmount = 789,
    AttrReprocessingSkillType = 790,
    AttrScanAnalyzeCount = 791,
    AttrControlTowerMissileVelocityBonus = 792,
    AttrShipBonusPirateFaction = 793,
    AttrProbesInGroup = 794,
    AttrShipBonus1ABC = 795,
    AttrMassAddition = 796,     // this is solely for mwd/ab mass addition. (checked in client)
    AttrMaximumRangeCap = 797,
    AttrEntityBracketColour = 798,
    AttrImplantSetBloodraider = 799,
    AttrContrabandDetectionChanceBonus = 800,
    AttrDeadspaceUnsafe = 801,
    AttrImplantSetSerpentis = 802,
    AttrImplantSetSerpentis2 = 803,
    AttrEliteBonusInterceptor2 = 804,
    AttrQuantity = 805,
    AttrRepairBonus = 806,
    AttrEliteBonusIndustrial1 = 807,
    AttrEliteBonusIndustrial2 = 808,
    AttrShipBonus2AI = 809,
    AttrShipBonus2CI = 811,
    AttrShipBonus2GI = 813,
    AttrShipBonus2MI = 814,
    AttrPropulsionFusionStrengthBonus = 815,
    AttrPropulsionIonStrengthBonus = 816,
    AttrPropulsionMagpulseStrengthBonus = 817,
    AttrPropulsionPlasmaStrengthBonus = 818,
    AttrPropulsionFusionStrength = 819,
    AttrPropulsionIonStrength = 820,
    AttrPropulsionMagpulseStrength = 821,
    AttrPropulsionPlasmaStrength = 822,
    AttrHitsMissilesOnly = 823,                         //smartbombs
    AttrEwStrengthBonus = 828,
    AttrPropulsionSkillPropulsionStrengthBonus = 829,
    AttrBonusComplexAngel10 = 830,
    AttrEwTargetJam = 831,
    AttrEwTargetPaintStrengthBonus = 832,
    AttrCommandBonus = 833,
    AttrWingCommandBonus = 834,
    AttrStealthBomberLauncherPower = 837,
    AttrImplantSetGuristas = 838,
    AttrEliteBonusCoverOps2 = 839,
    AttrAgentID = 840,
    AttrAgentCommRange = 841,
    AttrReactionGroup1 = 842,
    AttrReactionGroup2 = 843,
    AttrAgentAutoPopupRange = 844,
    AttrHiddenLauncherDamageBonus = 845,
    AttrScanStrengthBonus = 846,                        // skill, implant, launcher, rig.  in %
    AttrAoeVelocityBonus = 847,
    AttrAoeCloudSizeBonus = 848,
    AttrCanUseCargoInSpace = 849,
    AttrSquadronCommandBonus = 850,
    AttrShieldBoostCapacitorBonus = 851,
    AttrSiegeModeWarpStatus = 852,
    AttrAdvancedAgility = 853,
    AttrDisallowAssistance = 854,
    AttrActivationTargetLoss = 855,
    AttrAoeFalloffBonus = 857,
    AttrMissileEntityAoeCloudSizeMultiplier = 858,
    AttrMissileEntityAoeVelocityMultiplier = 859,
    AttrMissileEntityAoeFalloffMultiplier = 860,
    AttrCanJump = 861,
    AttrUsageWeighting = 862,
    AttrImplantSetAngel = 863,
    AttrImplantSetSansha = 864,
    AttrPlanetAnchorDistance = 865,
    AttrJumpDriveConsumptionType = 866,
    AttrJumpDriveRange = 867,
    AttrJumpDriveConsumptionAmount = 868,
    AttrJumpDriveDuration = 869,
    AttrJumpDriveRangeBonus = 870,
    AttrJumpDriveDurationBonus = 871,
    AttrDisallowOffensiveModifiers = 872,
    AttrAdvancedCapitalAgility = 874,
    AttrDreadnoughtShipBonusA1 = 875,
    AttrDreadnoughtShipBonusA2 = 876,
    AttrDreadnoughtShipBonusC1 = 877,
    AttrDreadnoughtShipBonusC2 = 878,
    AttrDreadnoughtShipBonusG1 = 879,
    AttrDreadnoughtShipBonusG2 = 880,
    AttrDreadnoughtShipBonusM1 = 881,
    AttrDreadnoughtShipBonusM2 = 882,
    AttrMindlinkBonus = 884,
    AttrConsumptionQuantityBonus = 885,
    AttrFreighterBonusA1 = 886,
    AttrFreighterBonusA2 = 887,
    AttrFreighterBonusC1 = 888,
    AttrFreighterBonusC2 = 889,
    AttrFreighterBonusG2 = 890,
    AttrFreighterBonusG1 = 891,
    AttrFreighterBonusM1 = 892,
    AttrFreighterBonusM2 = 893,
    AttrSpeedBoostBonus = 894,
    AttrArmorDamageAmountBonus = 895,
    AttrArmorDamageDurationBonus = 896,
    AttrShieldBonusDurationBonus = 897,
    AttrJumpDriveCapacitorNeed = 898,
    AttrJumpDriveCapacitorNeedBonus = 899,
    AttrAccessDifficulty = 901,
    AttrAccessDifficultyBonus = 902,
    AttrSpawnWithoutGuardsToo = 903,
    AttrWarcruiserCPUBonus = 904,
    AttrTacklerBonus = 905,
    AttrDisallowEarlyDeactivation = 906,        // not used, but would be helpful
    AttrHasShipMaintenanceBay = 907,            //flagShipHangar  AttrShipMaintenanceBayCapacity
    AttrShipMaintenanceBayCapacity = 908,
    AttrMaxShipGroupActiveID = 909,
    AttrMaxShipGroupActive = 910,
    AttrHasCorporateHangars = 911,              //flagHangar, flagCorpHangar2-7  AttrCorporateHangarCapacity
    AttrCorporateHangarCapacity = 912,
    AttrGallenteNavyBonus = 913,
    AttrGallenteNavyBonusMultiplier = 914,
    AttrCaldariNavyBonus = 915,
    AttrCaldariNavyBonusMultiplier = 916,
    AttrAmarrNavyBonus = 917,
    AttrAmarrNavyBonusMulitplier = 918,
    AttrRepublicFleetBonus = 919,
    AttrRepublicFleetBonusMultiplier = 920,
    AttrOreCompression = 921,
    AttrEliteBonusBarge1 = 924,
    AttrEliteBonusBarge2 = 925,
    AttrShipBonus3ORE = 926,
    AttrMiningUpgradeCPUReductionBonus = 927,
    AttrEntityTargetJam = 928,
    AttrEntityTargetJamDuration = 929,
    AttrEntityTargetJamDurationChance = 930,            // npcActivationChanceAttributeID in dgmEffects
    AttrEntityCapacitorDrainDurationChance = 931,       // npcActivationChanceAttributeID in dgmEffects
    AttrEntitySensorDampenDurationChance = 932,         // npcActivationChanceAttributeID in dgmEffects
    AttrEntityTrackingDisruptDurationChance = 933,      // npcActivationChanceAttributeID in dgmEffects
    AttrEntityTargetPaintDurationChance = 935,          // npcActivationChanceAttributeID in dgmEffects
    AttrEntityTargetJamMaxRange = 936,
    AttrEntityCapacitorDrainMaxRange = 937,
    AttrEntitySensorDampenMaxRange = 938,
    AttrEntityTrackingDisruptMaxRange = 940,
    AttrEntityTargetPaintMaxRange = 941,
    AttrEntityCapacitorDrainDuration = 942,
    AttrEntitySensorDampenDuration = 943,
    AttrEntityTrackingDisruptDuration = 944,
    AttrEntityTargetPaintDuration = 945,
    AttrEntityCapacitorDrainAmount = 946,
    AttrEntitySensorDampenMultiplier = 947,
    AttrEntityTrackingDisruptMultiplier = 948,
    AttrEntityTargetPaintMultiplier = 949,
    AttrEntitySensorDampenFallOff = 950,
    AttrEntityTrackingDisruptFallOff = 951,
    AttrEntityCapacitorFallOff = 952,
    AttrEntityTargetJamFallOff = 953,
    AttrEntityTargetPaintFallOff = 954,
    AttrIsCaldariNavy = 955,
    AttrDamageModifierMultiplierBonus = 956,
    AttrCNavyModOncNavyShip = 957,
    AttrHardeningBonus = 958,
    AttrEntityShieldBoostLargeDelayChance = 959,
    AttrCaldariNavyBonusMultiplier2 = 960,
    AttrCaldarNavyBonus2 = 961,
    AttrEliteBonusReconShip1 = 962,
    AttrEliteBonusReconShip2 = 963,
    AttrPassiveEmDamageResonanceMultiplier = 964,
    AttrPassiveThermalDamageResonanceMultiplier = 965,
    AttrPassiveKineticDamageResonanceMultiplier = 966,
    AttrPassiveExplosiveDamageResonanceMultiplier = 967,
    AttrHasStasisWeb = 968,
    AttrActiveEmDamageResonance = 969,
    AttrActiveThermalDamageResonance = 970,
    AttrActiveKineticDamageResonance = 971,
    AttrActiveExplosiveDamageResonance = 972,
    AttrSignatureRadiusBonusPercent = 973,
    AttrHullEmDamageResonance = 974,
    AttrHullExplosiveDamageResonance = 975,
    AttrHullKineticDamageResonance = 976,
    AttrHullThermalDamageResonance = 977,
    AttrMaxGroupOnline = 978,
    AttrMaxJumpClones = 979,
    AttrHasCloneJumpSlots = 980,
    AttrAllowsCloneJumpsWhenActive = 981,
    AttrCanReceiveCloneJumps = 982,
    AttrSignatureRadiusAdd = 983,
    AttrEmDamageResistanceBonus = 984,
    AttrExplosiveDamageResistanceBonus = 985,
    AttrKineticDamageResistanceBonus = 986,
    AttrThermalDamageResistanceBonus = 987,
    AttrHardeningbonus2 = 988,
    AttrVolumePostPercent = 989,
    AttrActiveEmResistanceBonus = 990,
    AttrActiveExplosiveResistanceBonus = 991,
    AttrActiveThermicResistanceBonus = 992,
    AttrActiveKineticResistanceBonus = 993,
    AttrPassiveEmDamageResistanceBonus = 994,
    AttrPassiveExplosiveDamageResistanceBonus = 995,
    AttrPassiveKineticDamageResistanceBonus = 996,
    AttrPassiveThermicDamageResistanceBonus = 997,
    AttrIsRAMcompatible = 998,
    AttrEliteBonusCommandShips2 = 999,
    AttrEliteBonusCommandShips1 = 1000,
    AttrJumpPortalConsumptionMassFactor = 1001,
    AttrJumpPortalDuration = 1002,
    AttrEliteBonusCommandShip1DONOTUSE = 1003,
    AttrEliteBonusCommandShip2DONOTUSE = 1004,
    AttrJumpPortalCapacitorNeed = 1005,
    AttrEntityShieldBoostDelayChanceSmall = 1006,       // npcActivationChanceAttributeID in dgmEffects
    AttrEntityShieldBoostDelayChanceMedium = 1007,      // npcActivationChanceAttributeID in dgmEffects
    AttrEntityShieldBoostDelayChanceLarge = 1008,       // npcActivationChanceAttributeID in dgmEffects
    AttrEntityArmorRepairDelayChanceSmall = 1009,       // npcActivationChanceAttributeID in dgmEffects
    AttrEntityArmorRepairDelayChanceMedium = 1010,      // npcActivationChanceAttributeID in dgmEffects
    AttrEntityArmorRepairDelayChanceLarge = 1011,       // npcActivationChanceAttributeID in dgmEffects
    AttrEliteBonusInterdictors1 = 1012,
    AttrEliteBonusInterdictors2 = 1013,
    AttrDisallowRepeatingActivation = 1014,
    AttrEntityShieldBoostDelayChanceSmallMultiplier = 1015,
    AttrEntityShieldBoostDelayChanceMediumMultiplier = 1016,
    AttrEntityShieldBoostDelayChanceLargeMultiplier = 1017,
    AttrEntityArmorRepairDelayChanceSmallMultiplier = 1018,
    AttrEntityArmorRepairDelayChanceMediumMultiplier = 1019,
    AttrEntityArmorRepairDelayChanceLargeMultiplier = 1020,
    AttrWarpAccuracyMaxRange = 1021,
    AttrWarpAccuracyFactor = 1022,
    AttrWarpAccuracyFactorMultiplier = 1023,
    AttrWarpAccuracyMaxRangeMultiplier = 1024,
    AttrWarpAccuracyFactorPercentage = 1025,
    AttrWarpAccuracyMaxRangePercentage = 1026,
    AttrScanGravimetricStrengthPercent = 1027,
    AttrScanLadarStrengthPercent = 1028,
    AttrScanMagnetometricStrengthPercent = 1029,
    AttrScanRadarStrengthPercent = 1030,
    AttrControlTowerSize = 1031,
    AttrAnchoringSecurityLevelMax = 1032,
    AttrAnchoringRequiresSovereignty = 1033,
    AttrCovertOpsAndReconOpsCloakModuleDelay = 1034,
    AttrCovertOpsStealthBomberTargettingDelay = 1035,
    AttrTitanAmarrBonus1 = 1036,
    AttrTitanAmarrBonus2 = 1038,
    AttrTitanCaldariBonus1 = 1039,
    AttrTitanCaldariBonus2 = 1040,
    AttrTitanGallenteBonus1 = 1041,
    AttrTitanGallenteBonus2 = 1042,
    AttrTitanMinmatarBonus1 = 1043,
    AttrTitanMinmatarBonus2 = 1044,
    AttrMaxTractorVelocity = 1045,
    AttrCanNotBeTrainedOnTrial = 1047,
    AttrDisallowOffensiveModifierBonus = 1048,
    AttrCarrierAmarrBonus1 = 1049,
    AttrCarrierAmarrBonus2 = 1050,
    AttrCarrierAmarrBonus3 = 1051,
    AttrCarrierCaldariBonus1 = 1052,
    AttrCarrierCaldariBonus2 = 1053,
    AttrCarrierCaldariBonus3 = 1054,
    AttrCarrierGallenteBonus1 = 1055,
    AttrCarrierGallenteBonus2 = 1056,
    AttrCarrierGallenteBonus3 = 1057,
    AttrCarrierMinmatarBonus1 = 1058,
    AttrCarrierMinmatarBonus2 = 1059,
    AttrCarrierMinmatarBonus3 = 1060,
    AttrTitanAmarrBonus3 = 1061,
    AttrTitanAmarrBonus4 = 1062,
    AttrTitanCaldariBonus3 = 1063,
    AttrTitanCaldariBonus4 = 1064,
    AttrTitanGallenteBonus3 = 1065,
    AttrTitanGallenteBonus4 = 1066,
    AttrTitanMinmatarBonus4 = 1067,
    AttrTitanMinmatarBonus3 = 1068,
    AttrCarrierAmarrBonus4 = 1069,
    AttrCarrierCaldariBonus4 = 1070,
    AttrCarrierGallenteBonus4 = 1071,
    AttrCarrierMinmatarBonus4 = 1072,
    AttrMaxJumpClonesBonus = 1073,
    AttrDisallowInEmpireSpace = 1074,
    AttrMissileNeverDoesDamage = 1075,
    AttrImplantBonusVelocity = 1076,
    AttrMaxDCUModules = 1077,
    AttrCapacitorCapacityBonus = 1079,
    AttrCpuPenaltySuperWeapon = 1080,
    AttrCpuBonusSuperWeapon = 1081,
    AttrCpuPenaltyPercent = 1082,
    AttrArmorHpBonus2 = 1083,
    AttrVelocityBonus2 = 1084,
    AttrHasFuelCargo = 1085,                    // flagFuelBay     AttrFuelCargoCapacity
    AttrFuelCargoCapacity = 1086,
    AttrBoosterness = 1087,
    AttrExpiryTime = 1088,
    AttrBoosterEffectChance1 = 1089,                    //fittingUsageChanceAttributeID in dgmEffects table
    AttrBoosterEffectChance2 = 1090,                    //fittingUsageChanceAttributeID in dgmEffects table
    AttrBoosterEffectChance3 = 1091,                    //fittingUsageChanceAttributeID in dgmEffects table
    AttrBoosterEffectChance4 = 1092,                    //fittingUsageChanceAttributeID in dgmEffects table
    AttrBoosterEffectChance5 = 1093,                    //fittingUsageChanceAttributeID in dgmEffects table
    AttrDisplayCapacitorCapacityBonus = 1094,
    AttrDisplayShieldBoostMultiplier = 1095,
    AttrDisplayShieldCapacityBonus = 1096,
    AttrDisplayAoeVelocityBonus = 1097,
    AttrDisplayRangeSkillBonus = 1098,
    AttrBoosterAttribute1 = 1099,
    AttrBoosterAttribute2 = 1100,
    AttrBoosterAttribute3 = 1101,
    AttrBoosterAttribute4 = 1102,
    AttrBoosterAttribute5 = 1103,
    AttrDisplayMaxVelocityBonus = 1104,
    AttrDisplayArmorHpBonus = 1105,
    AttrDisplayMissileMaxVelocityBonus = 1106,
    AttrDisplayArmorDamageAmountBonus = 1107,
    AttrDisplayFalloffModifier = 1108,
    AttrDisplayTrackingSpeedModifier = 1109,
    AttrDisplayAoeCloudsizeModifier = 1110,
    AttrDisplayMaxRangeModifier = 1111,
    AttrInventionPropabilityMultiplier = 1112,
    AttrInventionMEModifier = 1113,
    AttrInventionPEModifier = 1114,
    AttrDecryptorID = 1115,
    AttrScanProbeStrength = 1116,
    AttrScanStrengthSignatures = 1117,
    AttrScanStrengthDronesProbes = 1118,
    AttrScanStrengthScrap = 1119,
    AttrScanStrengthShips = 1120,
    AttrScanStrengthStructures = 1121,
    AttrMaxScanGroups = 1122,
    AttrScanDuration = 1123,
    AttrInventionMaxRunModifier = 1124,         // invention shit
    AttrBoosterChanceBonus = 1125,
    AttrBoosterAttributeModifier = 1126,
    AttrInterfaceID = 1127,
    AttrDatacore1ID = 1128,
    AttrDatacore2ID = 1129,
    AttrEcmStrengthBonusPercent = 1130,
    AttrMassBonusPercentage = 1131,
    AttrUpgradeCapacity = 1132,
    AttrEntityMaxVelocitySignatureRadiusMultiplier = 1133,
    AttrMaxTargetRangeMultiplierSet = 1134,
    AttrScanResolutionMultiplierSet = 1135,
    AttrScanAllStrength = 1136,                         //Unknown Site  (scanning/exploration)
    AttrRigSlots = 1137,
    AttrDrawback = 1138,
    AttrRigDrawbackBonus = 1139,
    AttrBoosterArmorHPPenalty = 1141,
    AttrBoosterArmorRepairAmountPenalty = 1142,
    AttrBoosterShieldCapacityPenalty = 1143,
    AttrBoosterTurretOptimalRange = 1144,
    AttrBoosterTurretTrackingPenalty = 1145,
    AttrBoosterTurretFalloffPenalty = 1146,
    AttrBoosterAOEVelocityPenalty = 1147,
    AttrBoosterMissileVelocityPenalty = 1148,
    AttrBoosterMissileAOECloudPenalty = 1149,
    AttrBoosterCapacitorCapacityPenalty = 1150,
    AttrBoosterMaxVelocityPenalty = 1151,
    AttrUpgradeLoad = 1152,
    AttrUpgradeCost = 1153,
    AttrUpgradeSlotsLeft = 1154,
    AttrResearchPointCost = 1155,
    AttrMaxScanDeviationModifier = 1156,
    AttrCommandBonus2 = 1157,
    AttrUntargetable = 1158,
    AttrArmorHPBonusAdd = 1159,
    AttrAccessDifficultyBonusModifier = 1160,
    AttrScanFrequencyResult = 1161,
    AttrExplosionDelayWreck = 1162,
    AttrCanCloak = 1163,
    AttrSpeedFactorBonus = 1164,
    AttrControlTowerMinimumDistance = 1165,
    AttrPosPlayerControlStructure = 1167,
    AttrIsIncapacitated = 1168,
    AttrScanGenericStrength = 1169,
    AttrStructureArmorRepairAmount = 1170,              //typeID 27676
    AttrStructureShieldRepairAmount = 1171,             //typeID 27676
    AttrStructureArmorBoostValue = 1172,                // not used
    AttrStructureShieldBoostValue = 1173,               // not used
    AttrPosStructureControlAmount = 1174,               // not used
    AttrHeatHi = 1175,                                  // ship attrib
    AttrHeatMed = 1176,                                 // ship attrib
    AttrHeatLow = 1177,                                 // ship attrib
    AttrHeatCapacityHi = 1178,                          // ship attrib
    AttrHeatDissipationRateHi = 1179,                   // ship attrib
    AttrHeatAbsorbtionRateModifier = 1180,              // module attrib      defines how much extra heat this module will absorb
    AttrOverloadDurationBonus = 1181,
    AttrHeatAbsorbtionRateHi = 1182,                    // not used
    AttrHeatAbsorbtionRateMed = 1183,                   // not used
    AttrHeatAbsorbtionRateLow = 1184,                   // not used
    AttrOnliningRequiresSovereigntyLevel = 1185,
    AttrRemoteArmorDamageAmountBonus = 1186,
    AttrRemoteArmorDamageDurationBonus = 1187,
    AttrShieldTransportDurationBonus = 1188,
    AttrShieldTransportAmountBonus = 1189,
    AttrEwCapacitorNeedBonus = 1190,
    AttrMaxDronePercentageBonus = 1191,
    AttrTriageCpuNeedBonus = 1192,
    AttrProjECMDurationBonus = 1193,
    AttrProjECMCpuNeedBonus = 1194,
    AttrPosAnchoredPerSolarSystemAmount = 1195,
    AttrHeatDissipationRateMed = 1196,         // ship attrib
    AttrHeatDissipationRateLow = 1198,         // ship attrib
    AttrHeatCapacityMed = 1199,         // ship attrib
    AttrHeatCapacityLow = 1200,         // ship attrib
    AttrRemoteHullDamageAmountBonus = 1201,
    AttrRemoteHullDamageDurationBonus = 1202,
    AttrPowerTransferAmountBonus = 1203,
    AttrPowerTransferDurationBonus = 1204,
    AttrOverloadRofBonus = 1205,
    AttrOverloadSelfDurationBonus = 1206,
    AttrIsGlobal = 1207,
    AttrOverloadHardeningBonus = 1208,
    AttrBombDeploymentCpuNeedMultiplier = 1209,
    AttrOverloadDamageModifier = 1210,
    AttrHeatDamage = 1211,                          // module attrib
    AttrRequiredThermoDynamicsSkill = 1212,         // module attrib
    AttrHeatDamageBonus = 1213,                     // module attrib
    AttrPosStructureControlDistanceMax = 1214,
    AttrAnchoringRequiresSovereigntyLevel = 1215,
    AttrShieldTransportCpuNeedBonus = 1216,
    AttrRemoteArmorPowerNeedBonus = 1217,
    AttrPowerTransferPowerNeedBonus = 1218,
    AttrDroneArmorDamageAmountBonus = 1219,
    AttrDroneShieldBonusBonus = 1220,
    AttrJumpDelayDuration = 1221,
    AttrOverloadRangeBonus = 1222,
    AttrOverloadSpeedFactorBonus = 1223,
    AttrHeatGenerationMultiplier = 1224,            // ship attrib
    AttrOverloadECMStrengthBonus = 1225,
    AttrOverloadECCMStrenghtBonus = 1226,
    AttrSignatureRadiusBonusBonus = 1227,
    AttrSignatureRadiusMultiplierMultiplier = 1228,
    AttrThermodynamicsHeatDamage = 1229,            // skill attrib
    AttrOverloadArmorDamageAmount = 1230,
    AttrOverloadShieldBonus = 1231,
    AttrLeechBalanceFactor = 1232,
    AttrSecondaryCapacity = 1233,
    AttrSurveyScannerRangeBonus = 1234,
    AttrCargoScannerRangeBonus = 1235,
    AttrCommandBonusEffective = 1236,
    AttrCommandBonusAdd = 1237,
    AttrCommandBonusEffectiveAdd = 1238,
    AttrShipBonus1ORECapital = 1239,
    AttrShipBonus2ORECapital = 1240,
    AttrShipBonus3ORECapital = 1243,
    AttrShipBonus4ORECapital = 1244,
    AttrDisallowActivateOnWarp = 1245,
    AttrEliteBonusHeavyInterdictors1 = 1246,
    AttrEliteBonusHeavyInterdictors2 = 1247,
    AttrEliteBonusElectronicAttackShip1 = 1249,
    AttrEliteBonusElectronicAttackShip2 = 1250,
    AttrSecurityClearance = 1251,
    AttrIsCovert = 1252,
    AttrJumpHarmonics = 1253,
    AttrCanNotUseStargates = 1254,
    AttrDroneDamageBonus = 1255,
    AttrDroneHPBonus = 1256,
    AttrEliteBonusBlackOps1 = 1257,
    AttrEliteBonusBlackOps2 = 1258,
    AttrHeatAttenuationHi = 1259,
    AttrHeatAttenuationMed = 1261,
    AttrHeatAttenuationLow = 1262,
    AttrTowerHPOnlineMutator = 1263,
    AttrBrokenRepairCostMultiplier = 1264,
    AttrEliteBonusViolators1 = 1265,
    AttrEliteBonusViolators2 = 1266,
    AttrModuleRepairRate = 1267,
    AttrEliteBonusViolatorsRole1 = 1268,
    AttrEliteBonusViolatorsRole2 = 1269,
    AttrSpeedBoostFactorBonus = 1270,
    AttrDroneBandwidth = 1271,
    AttrDroneBandwidthUsed = 1272,
    AttrDroneBandwidthLoad = 1273,
    AttrMiningTargetMultiplier = 1274,
    AttrDroneIsAgressive = 1275,
    AttrNonBrokenModuleRepairCostMultiplier = 1276,
    AttrShipBrokenModuleRepairCostMultiplier = 1277,
    AttrDroneIsChaotic = 1278,
    AttrEliteBonusViolatorsRole3 = 1279,
    AttrEliteBonusInterceptorRole = 1280,
    AttrBaseWarpSpeed = 1281,
    AttrImplantSetThukker = 1282,
    AttrFightersAttackAndFollow = 1283,
    AttrImplantSetSisters = 1284,
    AttrRequiredSkill4 = 1285,
    AttrRequiredSkill4Level = 1286,
    AttrRequiredSkill5Level = 1287,
    AttrRequiredSkill6Level = 1288,
    AttrRequiredSkill5 = 1289,
    AttrRequiredSkill6 = 1290,
    AttrImplantSetSyndicate = 1291,
    AttrImplantSetORE = 1292,
    AttrImplantSetMordus = 1293,
    AttrShipBrokenRepairCostMultiplierBonus = 1294,
    AttrModuleRepairRateBonus = 1295,
    AttrConsumptionQuantityBonusPercentage = 1296,
    AttrDroneFocusFire = 1297,
    AttrCanFitShipGroup1 = 1298,
    AttrCanFitShipGroup2 = 1299,
    AttrCanFitShipGroup3 = 1300,
    AttrCanFitShipGroup4 = 1301,
    AttrCanFitShipType1 = 1302,
    AttrCanFitShipType2 = 1303,
    AttrCanFitShipType3 = 1304,
    AttrCanFitShipType4 = 1305,
    AttrMaxRangeMultiplierBonusAdditive = 1306,
    AttrTrackingSpeedMultiplierBonusAdditive = 1307,
    AttrMaxTargetRangeMultiplierBonusAdditive = 1308,
    AttrScanResolutionMultiplierBonusAdditive = 1309,
    AttrCommandBonusHidden = 1310,
    AttrEliteBonusJumpFreighter1 = 1311,
    AttrEliteBonusJumpFreighter2 = 1312,
    AttrMaxTargetRangeBonusBonus = 1313,
    AttrScanResolutionBonusBonus = 1314,
    AttrMaxRangeBonusBonus = 1315,
    AttrTrackingSpeedBonusBonus = 1316,
    AttrMaxRangeHidden = 1317,
    AttrWarpScrambleStrengthHidden = 1318,
    AttrCapacitorNeedHidden = 1319,
    AttrCommandBonusECM = 1320,
    AttrCommandBonusRSD = 1321,
    AttrCommandBonusTD = 1322,
    AttrCommandBonusTP = 1323,
    AttrMassBonusPercentageBonus = 1324,
    AttrSpeedBoostFactorBonusBonus = 1325,
    AttrSpeedFactorBonusBonus = 1326,
    AttrWarpScrambleRangeBonus = 1327,
    AttrDroneBandwidthMultiplier = 1328,
    AttrDroneBandwidthBonusAdd = 1329,
    AttrIsHacking = 1330,
    AttrIsArcheology = 1331,
    AttrFalloffBonusBonus = 1332,
    AttrMaxVelocityLimited = 1333,
    AttrMaxVelocityActivationLimit = 1334,
    AttrDefenderRaceID = 1335,
    AttrJumpClonesLeft = 1336,
    AttrCaptureProximityRange = 1337,
    AttrFactionDefenderID = 1339,
    AttrFactionOffenderID = 1340,
    AttrFactionID = 1341,
    AttrActivationBlocked = 1349,
    AttrActivationBlockedStrenght = 1350,
    AttrPosCargobayAcceptType = 1351,
    AttrPosCargobayAcceptGroup = 1352,
    AttrAoeDamageReductionFactor = 1353,
    AttrAoeDamageReductionSensitivity = 1354,
    AttrShipOrcaTractorBeamRangeBonus1 = 1355,
    AttrShipOrcaCargoBonusOrca1 = 1356,
    AttrShipOrcaTractorBeamVelocityBonus2 = 1357,
    AttrShipOrcaForemanBonus = 1358,
    AttrShipOrcaSurveyScannerBonus = 1359,
    AttrShipBonus1HPExtender = 1360,
    AttrEliteIndustrialCovertCloakBonus = 1361,
    AttrSubSystemSlot = 1366,
    AttrMaxSubSystems = 1367,
    AttrTurretHardPointModifier = 1368,
    AttrLauncherHardPointModifier = 1369,
    AttrBaseScanRange = 1370,                           //probe attrib.   in AU
    AttrBaseSensorStrength = 1371,                      //probe attrib.   unknown uom
    AttrBaseMaxScanDeviation = 1372,                    //probe attrib.   in % of current scan range
    AttrRangeFactor = 1373,                             //probe attrib.   integer.  multiplication factor for getting sensor str at range
    AttrHiSlotModifier = 1374,
    AttrMedSlotModifier = 1375,
    AttrLowSlotModifier = 1376,
    AttrCpuOutputAdd = 1377,
    AttrPowerOutputAdd = 1378,
    AttrMaxVelocityAdd = 1379,
    AttrFitsToShipType = 1380,
    AttrWormholeTargetSystemClass = 1381,
    AttrWormholeMaxStableTime = 1382,
    AttrWormholeMaxStableMass = 1383,
    AttrWormholeMassRegeneration = 1384,
    AttrWormholeMaxJumpMass = 1385,
    AttrWormholeTargetRegion1 = 1386,
    AttrWormholeTargetRegion2 = 1387,
    AttrWormholeTargetRegion3 = 1388,
    AttrWormholeTargetRegion4 = 1389,
    AttrWormholeTargetRegion5 = 1390,
    AttrWormholeTargetRegion6 = 1391,
    AttrWormholeTargetRegion7 = 1392,
    AttrWormholeTargetRegion8 = 1393,
    AttrWormholeTargetRegion9 = 1394,
    AttrWormholeTargetConstellation1 = 1395,
    AttrWormholeTargetConstellation2 = 1396,
    AttrWormholeTargetConstellation3 = 1397,
    AttrWormholeTargetConstellation4 = 1398,
    AttrWormholeTargetConstellation5 = 1399,
    AttrWormholeTargetConstellation6 = 1400,
    AttrWormholeTargetConstellation7 = 1401,
    AttrWormholeTargetConstellation8 = 1402,
    AttrWormholeTargetConstellation9 = 1403,
    AttrWormholeTargetSystem1 = 1404,
    AttrWormholeTargetSystem2 = 1405,
    AttrWormholeTargetSystem3 = 1406,
    AttrWormholeTargetSystem4 = 1407,
    AttrWormholeTargetSystem5 = 1408,
    AttrWormholeTargetSystem6 = 1409,
    AttrWormholeTargetSystem7 = 1410,
    AttrWormholeTargetSystem8 = 1411,
    AttrWormholeTargetSystem9 = 1412,
    AttrProbeCanScanShips = 1413,
    AttrEvasiveManeuverLevel = 1414,
    AttrTargetSwitchTimer = 1416,
    AttrColor = 1417,
    AttrPassiveArmorEmDamageResonance = 1418,
    AttrPassiveArmorThermalDamageResonance = 1419,
    AttrPassiveArmorKineticDamageResonance = 1420,
    AttrPassiveArmorExplosiveDamageResonance = 1421,
    AttrPassiveShieldExplosiveDamageResonance = 1422,
    AttrPassiveShieldEmDamageResonance = 1423,
    AttrPassiveShieldKineticDamageResonance = 1424,
    AttrPassiveShieldThermalDamageResonance = 1425,
    AttrPassiveHullEmDamageResonance = 1426,
    AttrPassiveHullExplosiveDamageResonance = 1427,
    AttrPassiveHullKineticDamageResonance = 1428,
    AttrPassiveHullThermalDamageResonance = 1429,
    AttrLightColor = 1430,
    AttrSubsystemBonusAmarrEngineering = 1431,
    AttrSubsystemBonusAmarrElectronic = 1432,
    AttrSubsystemBonusAmarrDefensive = 1433,
    AttrSubsystemBonusAmarrOffensive = 1434,
    AttrSubsystemBonusAmarrPropulsion = 1435,
    AttrSubsystemBonusGallenteEngineering = 1436,
    AttrSubsystemBonusGallenteElectronic = 1437,
    AttrSubsystemBonusGallenteDefensive = 1438,
    AttrSubsystemBonusGallenteOffensive = 1439,
    AttrSubsystemBonusGallentePropulsion = 1440,
    AttrSubsystemBonusCaldariEngineering = 1441,
    AttrSubsystemBonusCaldariElectronic = 1442,
    AttrSubsystemBonusCaldariDefensive = 1443,
    AttrSubsystemBonusCaldariOffensive = 1444,
    AttrSubsystemBonusCaldariPropulsion = 1445,
    AttrSubsystemBonusMinmatarEngineering = 1446,
    AttrSubsystemBonusMinmatarElectronic = 1447,
    AttrSubsystemBonusMinmatarDefensive = 1448,
    AttrSubsystemBonusMinmatarOffensive = 1449,
    AttrSubsystemBonusMinmatarPropulsion = 1450,
    AttrNpcAssistanceProirity = 1451,
    AttrNpcRemoteArmorRepairChance = 1453,              // npcActivationChanceAttributeID in dgmEffects
    AttrNpcRemoteArmorRepairDuration = 1454,
    AttrNpcRemoteArmorRepairAmount = 1455,
    AttrNpcRemoteArmorRepairThreshold = 1456,
    AttrWormholeTargetDistribution = 1457,
    AttrNpcRemoteShieldBoostDuration = 1458,
    AttrNpcRemoteShieldBoostChance = 1459,              // npcActivationChanceAttributeID in dgmEffects
    AttrNpcRemoteShieldBoostAmount = 1460,
    AttrNpcRemoteShieldBoostThreshold = 1462,
    AttrNpcAssistanceRange = 1464,
    AttrArmorEmDamageResistanceBonus = 1465,
    AttrArmorKineticDamageResistanceBonus = 1466,
    AttrArmorThermalDamageResistanceBonus = 1467,
    AttrArmorExplosiveDamageResistanceBonus = 1468,
    AttrMissileVelocityMultiplier = 1469,
    AttrMaxVelocityMultiplier = 1470,
    AttrMassMultiplier = 1471,
    AttrDroneRangeMultiplier = 1472,
    AttrScanGravimetricStrengthMultiplier = 1473,
    AttrScanLadarStrengthMultiplier = 1474,
    AttrScanMagnetometricStrengthMultiplier = 1475,
    AttrScanRadarStrengthMultiplier = 1476,
    AttrSignatureRadiusBonusMultiplier = 1477,
    AttrMaxTargetRangeBonusMultiplier = 1478,
    AttrScanResolutionBonusMultiplier = 1479,
    AttrTrackingSpeedBonusMultiplier = 1480,
    AttrMaxRangeBonusMultiplier = 1481,
    AttrDamageMultiplierMultiplier = 1482,
    AttrAoeVelocityMultiplier = 1483,
    AttrMaxDroneVelocityMultiplier = 1484,
    AttrHeatDamageMultiplier = 1485,            // beacon types (WH and Incursion)
    AttrOverloadBonusMultiplier = 1486,
    AttrEmpFieldRangeMultiplier = 1487,
    AttrSmartbombDamageMultiplier = 1488,
    AttrShieldEmDamageResistanceBonus = 1489,
    AttrShieldExplosiveDamageResistanceBonus = 1490,
    AttrShieldKineticDamageResistanceBonus = 1491,
    AttrShieldThermalDamageResistanceBonus = 1492,
    AttrSmallWeaponDamageMultiplier = 1493,
    AttrMediumWeaponDamageMultiplier = 1494,
    AttrArmorDamageAmountMultiplier = 1495,
    AttrShieldBonusMultiplier = 1496,
    AttrShieldBonusMultiplierRemote = 1497,
    AttrArmorDamageAmountMultiplierRemote = 1498,
    AttrCapacitorCapacityMultiplierSystem = 1499,
    AttrRechargeRateMultiplier = 1500,
    AttrNpcRemoteArmorRepairMaxTargets = 1501,
    AttrNpcRemoteShieldBoostMaxTargets = 1502,
    AttrShipBonusStrategicCruiserAmarr = 1503,
    AttrShipBonusStrategicCruiserCaldari = 1504,
    AttrShipBonusStrategicCruiserGallente = 1505,
    AttrShipBonusStrategicCruiserMinmatar = 1506,
    AttrSubsystemBonusAmarrDefensive2 = 1507,
    AttrSubsystemBonusAmarrElectronic2 = 1508,
    AttrSubsystemBonusAmarrEngineering2 = 1509,
    AttrSubsystemBonusCaldariOffensive2 = 1510,
    AttrSubsystemBonusAmarrOffensive2 = 1511,
    AttrSubsystemBonusAmarrPropulsion2 = 1512,
    AttrSubsystemBonusCaldariPropulsion2 = 1513,
    AttrSubsystemBonusCaldariElectronic2 = 1514,
    AttrSubsystemBonusCaldariEngineering2 = 1515,
    AttrSubsystemBonusCaldariDefensive2 = 1516,
    AttrSubsystemBonusGallenteDefensive2 = 1517,
    AttrSubsystemBonusGallenteElectronic2 = 1518,
    AttrSubsystemBonusGallenteEngineering2 = 1519,
    AttrSubsystemBonusGallentePropulsion2 = 1520,
    AttrSubsystemBonusGallenteOffensive2 = 1521,
    AttrSubsystemBonusMinmatarOffensive2 = 1522,
    AttrSubsystemBonusMinmatarPropulsion2 = 1523,
    AttrSubsystemBonusMinmatarElectronic2 = 1524,
    AttrSubsystemBonusMinmatarEngineering2 = 1525,
    AttrSubsystemBonusMinmatarDefensive2 = 1526,
    AttrArmorMaxDamageResonance = 1527,
    AttrShieldMaxDamageResonance = 1528,
    AttrHullMaxDamageResonance = 1529,
    AttrHullMaxDamageResonance1 = 1530,
    AttrSubsystemBonusAmarrOffensive3 = 1531,
    AttrSubsystemBonusGallenteOffensive3 = 1532,
    AttrSubsystemBonusCaldariOffensive3 = 1533,
    AttrSubsystemBonusMinmatarOffensive3 = 1534,
    AttrShipBonusCR3 = 1535,
    AttrEcmRangeBonus = 1536,
    AttrEliteBonusReconShips3 = 1537,
    AttrWarpBubbleImmune = 1538,
    AttrWarpBubbleImmuneModifier = 1539,
    AttrStealthBomberLauncherPower2 = 1540,
    AttrJumpHarmonicsModifier = 1541,
    AttrMaxGroupFitted = 1544,
    AttrDreadnoughtShipBonusM3 = 1545,
    AttrRigSize = 1547,
    AttrFuelBayCapacity = 1549,
    AttrImplantSetImperialNavy = 1550,
    AttrImplantSetCaldariNavy = 1552,
    AttrImplantSetFederationNavy = 1553,
    AttrImplantSetRepublicFleet = 1554,
    AttrFwLpKill = 1555,
    AttrOreHoldCapacity = 1556,
    AttrGasHoldCapacity = 1557,
    AttrMineralHoldCapacity = 1558,
    AttrSalvageHoldCapacity = 1559,
    AttrShipHoldCapacity = 1560,
    AttrSmallShipHoldCapacity = 1561,
    AttrMediumShipHoldCapacity = 1562,
    AttrLargeShipHoldCapacity = 1563,
    AttrIndustrialShipHoldCapacity = 1564,
    // these next 8 are implants
    AttrScanRadarStrengthModifier = 1565,
    AttrScanLadarStrengthModifier = 1566,
    AttrScanGravimetricStrengthModifier = 1567,
    AttrScanMagnetometricStrengthModifier = 1568,
    AttrImplantSetLGImperialNavy = 1569,
    AttrImplantSetLGFederationNavy = 1570,
    AttrImplantSetLGCaldariNavy = 1571,
    AttrImplantSetLGRepublicFleet = 1572,
    AttrAmmoHoldCapacity = 1573,
    AttrShipBonusATC1 = 1574,
    AttrShipBonusATC2 = 1575,
    AttrShipBonusATF1 = 1576,
    AttrShipBonusATF2 = 1577,
    AttreliteBonusCoverOps3 = 1578,
    AttreffectDeactivationDelay = 1579,
    AttrmaxDefenseBunkers = 1580,
    AttreliteBonusAssaultShips1 = 1581,
    AttrSpecialTutorialLootRespawnTime = 1582,
    AttrdevIndexMilitary = 1583,
    AttrdevIndexIndustrial = 1584,
    AttrdevIndexEconomic = 1585,
    AttrdevIndexResearchDevelopment = 1586,
    AttrsiegeModeGallenteDreadnoughtBonus2 = 1589,
    AttrAnchorDistanceMin = 1590,
    AttrAnchorDistanceMax = 1591,
    AttrtitanAmarrBonus5 = 1592,
    AttrtitanGallenteBonus5 = 1593,
    AttrtitanMinmatarBonus5 = 1594,
    AttranchoringRequiresSovUpgrade1 = 1595,
    AttrtitanCaldariBonus5 = 1596,
    AttrsovUpgradeSovereigntyHeldFor = 1597,
    AttrsovUpgradeBlockingUpgradeID = 1598,
    AttrsovUpgradeRequiredUpgradeID = 1599,
    AttrsovUpgradeRequiredOutpostUpgradeLevel = 1600,
    AttronliningRequiresSovUpgrade1 = 1601,
    AttrsovBillSystemCost = 1603,
    AttrdreadnoughtShipBonusC3 = 1605,
    AttrdistributionID_blood = 1606,
    AttrdistributionID_angel = 1607,
    AttrdistributionID_guristas = 1608,
    AttrdistributionID_serpentis = 1609,
    AttrdistributionID_drones = 1610,
    AttrdistributionID_sanshas=  1611,
    AttrreinforcementDuration = 1612,
    AttrreinforcementVariance = 1613,
    AttrdistributionID_mordus = 1614,
    AttrdevIndexSovereignty = 1615,
    AttrdistributionID =1616,
    AttrwebSpeedFactorBonus = 1619,
    AttrshipBonus3AF = 1623,
    AttrshipBonus3CF = 1624,
    AttrshipBonus3GF = 1625,
    AttrshipBonus3MF = 1626,
    /* these are for PI */
    AttrLogisticalCapacity = 1631,
    AttrPlanetRestriction = 1632,
    AttrPowerLoadPerKm = 1633,
    AttrCPULoadPerKm = 1634,
    AttrCPULoadLevelModifier = 1635,
    AttrPowerLoadLevelModifier = 1636,
    AttrImportTax = 1638,
    AttrExportTax = 1639,
    AttrImportTaxMultiplier = 1640,
    AttrExportTaxMultiplier = 1641,
    AttrPinExtractionQuantity = 1642,
    AttrPinCycleTime = 1643,
    AttrExtractorDepletionRange = 1644,
    AttrExtractorDepletionRate = 1645,
    AttrCommandCenterHoldCapacity = 1646,
    AttrBoosterMaxCharAgeHours = 1647,
    AttrAI_ShouldUseTargetSwitching = 1648,
    AttrAI_ShouldUseSecondaryTarget = 1649,
    AttrAI_ShouldUseSignatureRadius = 1650,
    AttrAI_ChanceToNotTargetSwitch = 1651,
    AttrAI_ShouldUseEffectMultiplier = 1652,
    AttrPlanetaryCommoditiesHoldCapacity = 1653,
    AttrAI_ImmuneToSuperWeapon = 1654,
    AttrAI_PreferredSignatureRadius = 1655,
    AttrAI_TankingModifierDrone = 1656,
    AttrAI_TankingModifier = 1657,
    AttrentityRemoteECMDuration = 1658,
    AttrentityRemoteECMMinDuration = 1659,
    AttrentityRemoteECMDurationScale = 1660,
    AttrentityRemoteECMBaseDuration = 1661,
    AttrentityRemoteECMExtraPlayerScale = 1662,
    AttrentityRemoteECMIntendedNumPlayers = 1663,
    AttrentityRemoteECMChanceOfActivation = 1664,       // npcUsageChanceAttributeID in dgmEffects
    AttrShipBonusOreIndustrial1 = 1669,
    AttrShipBonusOreIndustrial2 = 1670,
    AttrentityGroupShieldResistanceBonus = 1671,
    AttrentityGroupShieldResistanceDuration = 1672,
    AttrentityGroupShieldResistanceActivationChance = 1673,   // npcActivationChanceAttributeID in dgmEffects
    AttrentityGroupSpeedBonus = 1674,
    AttrentityGroupPropJamBonus = 1675,
    AttrentityGroupArmorResistanceBonus = 1676,
    AttrentityGroupSpeedDuration = 1677,
    AttrentityGroupSpeedActivationChance = 1678,        // npcActivationChanceAttributeID in dgmEffects
    AttrentityGroupPropJamDuration = 1679,
    AttrentityGroupPropJamActivationChance = 1680,      // npcActivationChanceAttributeID in dgmEffects
    AttrentityGroupArmorResistanceDuration = 1681,
    AttrentityGroupArmorResistanceActivationChance = 1682,   // npcActivationChanceAttributeID in dgmEffects
    /* these are for PI and may be calculated on-the-fly */
    AttrECUDecayFactor = 1683,
    AttrECUMaxVolume = 1684,
    AttrECUOverlapFactor = 1685,
    AttrECUNoiseFactor = 1687,
    AttrECUAreaOfInfluence = 1689,
    AttrECUExtractorHeadCPU = 1690,
    AttrECUExtractorHeadPower = 1691,
    /* end PI */
    AttrSystemEffectDamageReduction = 1686,
    AttrShipBonusPirateFaction2 = 1688,
    AttrmetaGroupID = 1692,
    AttrdistributionIDAngel01 = 1695,
    AttrdistributionIDAngel02 = 1696,
    AttrdistributionIDAngel03 = 1697,
    AttrdistributionIDAngel04 = 1698,
    AttrdistributionIDAngel05 = 1699,
    AttrdistributionIDAngel06 = 1700,
    AttrdistributionIDAngel07 = 1701,
    AttrdistributionIDAngel08 = 1702,
    AttrdistributionIDAngel09 = 1703,
    AttrdistributionIDAngel10 = 1704,
    AttrdistributionIDBlood01 = 1705,
    AttrdistributionIDBlood02 = 1706,
    AttrdistributionIDBlood03 = 1707,
    AttrdistributionIDBlood04 = 1708,
    AttrdistributionIDBlood05 = 1709,
    AttrdistributionIDBlood06 = 1710,
    AttrdistributionIDBlood07 = 1711,
    AttrdistributionIDBlood08 = 1712,
    AttrdistributionIDBlood09 = 1713,
    AttrdistributionIDBlood10 = 1714,
    AttrdistributionIDGurista01 = 1715,
    AttrdistributionIDGurista02 = 1716,
    AttrdistributionIDGurista03 = 1717,
    AttrdistributionIDGurista04 = 1718,
    AttrdistributionIDGurista05 = 1719,
    AttrdistributionIDGurista06 = 1720,
    AttrdistributionIDGurista07 = 1721,
    AttrdistributionIDGurista08 = 1722,
    AttrdistributionIDGurista09 = 1723,
    AttrdistributionIDGurista10 = 1724,
    AttrdistributionIDRogueDrone01 = 1725,
    AttrdistributionIDRogueDrone02 = 1726,
    AttrdistributionIDRogueDrone03 = 1727,
    AttrdistributionIDRogueDrone04 = 1728,
    AttrdistributionIDRogueDrone05 = 1729,
    AttrdistributionIDRogueDrone06 = 1730,
    AttrdistributionIDRogueDrone07 = 1731,
    AttrdistributionIDRogueDrone08 = 1732,
    AttrdistributionIDRogueDrone09 = 1733,
    AttrdistributionIDRogueDrone10 = 1734,
    AttrdistributionIDSansha01 = 1735,
    AttrdistributionIDSansha02 = 1736,
    AttrdistributionIDSansha03 = 1737,
    AttrdistributionIDSansha04 = 1738,
    AttrdistributionIDSansha05 = 1739,
    AttrdistributionIDSansha06 = 1740,
    AttrdistributionIDSansha07 = 1741,
    AttrdistributionIDSansha08 = 1742,
    AttrdistributionIDSansha09 = 1743,
    AttrdistributionIDSansha10 = 1744,
    AttrdistributionIDSerpentis01 = 1745,
    AttrdistributionIDSerpentis02 = 1746,
    AttrdistributionIDSerpentis03 = 1747,
    AttrdistributionIDSerpentis04 = 1748,
    AttrdistributionIDSerpentis05 = 1749,
    AttrdistributionIDSerpentis06 = 1750,
    AttrdistributionIDSerpentis07 = 1751,
    AttrdistributionIDSerpentis08 = 1752,
    AttrdistributionIDSerpentis09 = 1753,
    AttrdistributionIDSerpentis10 = 1754,
    AttrdistributionID01 = 1755,      /* search 2040 -  ore prospecting array */
    AttrdistributionID02 = 1756,
    AttrdistributionID03 = 1757,
    AttrdistributionID04 = 1758,
    AttrdistributionID05 = 1759,
    AttrdistributionID06 = 1760,
    AttrdistributionID07 = 1761,
    AttrdistributionID08 = 1762,
    AttrdistributionID09 = 1763,
    AttrdistributionID10 = 1764,
    AttrentityOverviewShipGroupId = 1766,
    AttrtypeColorScheme = 1768,
    AttrMaterialBayCapacity = 1770,
    AttrconstructionType = 1771,
    AttraccessDifficultyBonusAbsolutePercent = 1772,
    Attrgender = 1773,
    AttrconsumptionQuantityBonusPercent = 1775,
    AttrmanufactureCostBonusShowInfo = 1778,
    AttrnpcCustomsOfficeTaxRate = 1780,
    AttrdefaultCustomsOfficeTaxRate = 1781,
    AttrAllowedDroneGroup1 = 1782,
    AttrAllowedDroneGroup2 = 1783,
    AttrIsCapitalSize = 1785,
    AttrbcLargeTurretPower = 1786,
    AttrbcLargeTurretCPU = 1787,
    AttrbcLargeTurretCap = 1788,
    AttrbcSiegeMissileCPU = 1790,
    AttrbcSiegeMissilePower = 1791,
    AttrshipBonusBC3 = 1792,
    AttrshipBonusBC4 = 1793,
    AttrskillBonusBooster = 1794,
    AttrReloadTime = 1795,
    AttrclothingAlsoCoversCategory = 1797,
    AttrdisallowAgainstEwImmuneTarget = 1798,
    AttrimplantSetChristmas = 1799,
    AttrtriageRemoteModuleCapNeed = 1802,
    AttrMWDSignatureRadiusBonus = 1803,
    AttrQuafeHoldCapacity = 1804,
    // end of client const attrib data
    AttrrequiresSovereigntyDisplayOnly = 1806,
    AttrnosReflector = 1808,
    AttrneutReflector = 1809,
    AttrcapAttackReflector = 1811,
    AttrturretDamageScalingRadius = 1812,
    AttrtitanBonusScalingRadius = 1813,
    AttrnosReflectAmount = 1814,
    AttrneutReflectAmount = 1815,
    AttrneutReflectAmountBonus = 1816,
    AttrnosReflectAmountBonus = 1817,
    // end of bulkdata packets
    AttraurumConversionRate = 1818,
    AttrbaseDefenderAllyCost = 1820,
    AttrskillAllyCostModifierBonus = 1821,
    AttrrookieSETCapBonus = 1822,
    AttrrookieSETDamageBonus = 1823,
    AttrrookieWeaponDisruptionBonus = 1824,
    AttrrookieArmorResistanceBonus = 1825,
    AttrrookieSHTOptimalBonus = 1826,
    AttrrookieMissileKinDamageBonus = 1827,
    AttrrookieECMStrengthBonus = 1828,
    AttrrookieShieldResistBonus = 1829,
    AttrrookieSHTDamageBonus = 1830,
    AttrrookieDroneBonus = 1831,
    AttrrookieDampStrengthBonus = 1832,
    AttrrookieArmorRepBonus = 1833,
    AttrrookieTargetPainterStrengthBonus = 1834,
    AttrrookieShipVelocityBonus = 1835,
    AttrrookieSPTDamageBonus = 1836,
    AttrrookieShieldBoostBonus = 1837,
    AttrdamageDelayDuration = 1839,
    AttrCanFitShipGroup5 = 1872,
    AttrCanFitShipGroup6 = 1879,
    AttrCanFitShipGroup7 = 1880,
    AttrCanFitShipGroup8 = 1881,
    //  new shit from rhea below this line
    AttrMaxScanDeviationModifierModule = 1905,  // not used
    AttrScanDurationBonus = 1906,  // not used
    AttrScanStrengthBonusModule = 1907,  // not used
    AttrScanWormholeStrength = 1908,  // not used
    AttrDScanImmune = 1958      // not used, but neat idea
    /*
    warfareLinkCPUAdd   1882
    warfareLinkCPUPenalty   1883
    chargedArmorDamageMultiplier    1886
    shipBonusAD1    1887
    shipBonusAD2    1888
    shipBonusABC2   1889
    nondestructible     1890
    allowedInCapIndustrialMaintenanceBay    1891    ..deleted from TypeAttributes
    entityArmorRepairAmountPerSecond    1892
    entityShieldBoostAmountPerSecond    1893
    entityCapacitorLevel    1894
    entityCapacitorLevelModifierSmall   1895
    entityCapacitorLevelModifierMedium  1896
    entityCapacitorLevelModifierLarge   1897
    securityProcessingFee   1904
    maxScanDeviationModifierModule  1905
    scanDurationBonus   1906
    scanStrengthBonusModule     1907
    scanWormholeStrength    1908
    virusCoherence  1909
    virusStrength   1910
    virusElementSlots   1911
    spewContainerCount  1912
    defaultJunkLootTypeID   1913
    spewVelocity    1914
    virusCoherenceBonus     1915
    followsJumpClones   1916
    spewContainerLifeExtension  1917
    virusStrengthBonus  1918
    tierDifficulty  1919
    disallowActivateInForcefield    1920
    cloneJumpCoolDown   1921
    warfareLinkBonus    1922
    roleBonusMarauder   1923
    eliteBonusCommandShips3     1924
    piTaxReductionModifer   1925
    piTaxReduction  1926
    hackable    1927
    siphonRawMaterial   1928
    siphonProMaterial   1929
    siphonWasteAmount   1930
    implantSetWarpSpeed     1932
    siphonPolyMaterial  1933
    deactivateIfOffensive   1934
    overloadTrackingModuleStrengthBonus     1935
    overloadSensorModuleStrengthBonus   1936
    overloadPainterStrengthBonus    1937
    miningAmountBonusBonus  1938
    stationOreRefiningBonus     1939
    compressionTypeID   1940
    compressionQuantityNeeded   1941
    eliteBonusExpedition1   1942
    eliteBonusExpedition2   1943
    AttrCanFitShipType5 = 1944,
    nosOverride     1945
    anchoringSecurityLevelMin   1946
    roleBonusOverheatDST    1949
    warpSpeedAdd    1950
    industryStructureCostBonusSet   1951
    industryStructureCostBonus  1952
    industryJobCostMultiplier   1954
    industryBlueprintRank   1955
    clothingRemovesCategory     1956
    clothingRuleException   1957
    dscanImmune     1958
    inventionReverseEngineeringResearchSpeed    1959
    advancedIndustrySkillIndustryJobTimeBonus   1961
    energyWarfareStrengthMultiplier     1966
    aoeCloudSizeMultiplier  1967
    targetPainterStrengthMultiplier     1968
    stasisWebStrengthMultiplier     1969
    disallowInHighSec   1970
    jumpFatigueMultiplier   1971
    jumpThroughFatigueMultiplier    1972
    gateScrambleStatus  1973
    gateScrambleStrength    1974
    published   1975
    resistanceKiller    1978
    resistanceKillerHull    1979
    asteroidRadiusGrowthFactor  1980
    asteroidRadiusUnitSize  1981
    manufactureTimePerLevel     1982
    freighterBonusO1    1983
    freighterBonusO2    1984
    stanceSwitchTime    1985
    shipBonusTacticalDestroyerAmarr1    1986
    shipBonusTacticalDestroyerAmarr2    1987
    shipBonusTacticalDestroyerAmarr3    1988
    roleBonusTacticalDestroyer1     1989
    modeMaxRangePostDiv     1990
    modeMaxTargetRangePostDiv   1991
    modeRadarStrengthPostDiv    1992
    modeScanResPostDiv  1993
    modeLadarStrengthPercent    1994
    modeGravimetricStrengthPercent  1995
    modeMagnetometricStrengthPercent    1996
    modeEmResistancePostDiv     1997
    modeExplosiveResistancePostDiv  1998
    modeThermicResistancePostDiv    1999
    modeKineticResistancePostDiv    2000
    modeSignatureRadiusPostDiv  2001
    modeAgilityPostDiv  2002
    modeVelocityPostDiv     2003
    */
};

#endif // AttributeEnum_h__